/*
 * catalog/awt/OrderForm.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package catalog.awt;

import catalog.customer.Customer;
import catalog.customer.Catalog;
import catalog.net.Order;

import java.awt.*;
import java.io.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import javax.swing.border.*;

import smi.appConf.ApplicationConfiguration;
import smi.appConf.ActionConfiguration;
import smi.baseConf.BaseConfiguration;

import ssonet.SSONETConstants;

/**
* A dialog which displays an orderform
*/
public class OrderForm extends JDialog implements ActionListener {

    public Customer customer;
    public Catalog catalog;
    OrderSumPanel orderSumPanel;
    JFrame parentFrame;
    JButton okButton, cancelButton;
    float cost;

    /**
    * A panel with buttons for sending an order and cancelling of an order
    */
    class ButtonPanel extends JPanel {
	/**
	* creates the panel
        */
        public ButtonPanel() {
            GridBagLayout gridBagLayout = new GridBagLayout();
            GridBagConstraints gridBagConstraints = new GridBagConstraints();
            this.setLayout(gridBagLayout);
            gridBagConstraints.gridx = 0;
            gridBagConstraints.gridy = 0;
            gridBagConstraints.ipady = 10;
            gridBagConstraints.ipadx = 25;
            gridBagConstraints.weightx = 0.5;
            okButton = new JButton("Bestellung abschicken");
            gridBagLayout.setConstraints(okButton, gridBagConstraints);
            this.add(okButton);

            cancelButton = new JButton("Abbrechen");
            gridBagConstraints.gridx = 1;
            gridBagConstraints.gridy = 0;
            gridBagLayout.setConstraints(cancelButton, gridBagConstraints);
            this.add(cancelButton);
        }
    }

    /**
    * A panel with informations for the order
    */
    class InfoPanel extends JPanel {
	/**
	* creates the panel
        */
        public InfoPanel() {
	    BorderLayout bl = new BorderLayout();
	    this.setLayout(bl);
	    Panel p1 = new Panel();
	    GridLayout glHead = new GridLayout(2,1);
	    p1.setLayout(glHead);
	    this.add(p1, BorderLayout.NORTH);
	    Panel p2 = new Panel();
	    GridLayout glGoals = new GridLayout(2,4);
	    p2.setLayout(glGoals);
	    this.add(p2, BorderLayout.CENTER);
            Font font = new Font("Dialog", Font.BOLD,18);
            Label label = new Label("Informationen ber Sicherheit");
            label.setForeground(Color.red);
            label.setFont(font);
            p1.add(label);
            label = new Label("eigene Einstellungen beim Versenden:");
            p1.add(label);

            ActionConfiguration actionConfiguration =
		ApplicationConfiguration.getActionConfiguration(customer.name, OrderPanel.ORDER);
            label = new Label("Confidentiality:");
            p2.add(label);
            label = new Label();
            label.setForeground(Color.blue);
            label.setAlignment(Label.RIGHT);
            label.setText(actionConfiguration.toString(SSONETConstants.CONFIDENTIALITY));
            p2.add(label);
            label = new Label("Anonymity:");
            p2.add(label);
            label = new Label();
            label.setAlignment(Label.RIGHT);
            label.setForeground(Color.blue);
            label.setText(actionConfiguration.toString(SSONETConstants.ANONYMITY));
            p2.add(label);
            label = new Label("Integrity:");
            p2.add(label);
            label = new Label();
            label.setAlignment(Label.RIGHT);
            label.setForeground(Color.blue);
            label.setText(actionConfiguration.toString(SSONETConstants.INTEGRITY));
            p2.add(label);
            label = new Label("Accountability:");
            p2.add(label);
            label = new Label(actionConfiguration.toString(SSONETConstants.ACCOUNTABILITY_SIGN));
            label.setForeground(Color.blue);
            label.setAlignment(Label.RIGHT);
            p2.add(label);
        }
    }

    public TableModel dataModel;
    Vector data = new Vector(1);
    protected JTable tableView;

    private JScrollPane createTable() {
	final String[] Headlines={"Name",
	    "Nummer",
	    "Anzahl",
	    "Einheit",
	    "Preis pro Einheit",
	    "Gesamt"};
	dataModel = new AbstractTableModel() {
	    public int getColumnCount() {
		return Headlines.length;
	    }

	    public int getRowCount() {
		return data.size(); // liefert Anzahl der Vektoren in data
	    }

	    public Object getValueAt(int row, int col) {
		if (row<data.size()) {
		    return ( ((Vector)(data.elementAt(row))).elementAt(col));
		} else {
		    System.err.println("Error in Bestellungsformular.java: Zugriff auf nichtinitialisierte Tabellenzeile !\n");
		    return (new String("Error"));
		}
	    }

	    public String getColumnName(int column) {
		return Headlines[column];
	    }

	    public boolean isCellEditable(int row, int col) {
		return (false); // Keine Spalte soll editierbar sein
	    }
	};


	tableView = new JTable(dataModel);
	tableView.setAutoResizeMode(tableView.AUTO_RESIZE_ALL_COLUMNS);

	tableView.setRowSelectionAllowed(false);        // Zeilenauswahl aus
	tableView.setColumnSelectionAllowed(false);    // aber keine Spalten
	tableView.setCellSelectionEnabled(false);      // und keine einzelne Zelle
	tableView.getTableHeader().setReorderingAllowed(true);  // Spalten duerfen aber vertauscht werden

	DefaultTableCellRenderer renderer_one = new DefaultTableCellRenderer();
	renderer_one.setHorizontalAlignment(JLabel.RIGHT);
	tableView.getColumn(Headlines[4]).setCellRenderer(renderer_one);
	tableView.getColumn(Headlines[5]).setCellRenderer(renderer_one);
	tableView.getColumn(Headlines[3]).setCellRenderer(renderer_one);

	JScrollPane scrollpane = new JScrollPane(tableView);
	return scrollpane;
    } // End of createTable()


      /**
      * Creates the orderform.
    */
    public OrderForm(JFrame parentFrame, Customer customer, Catalog catalog, Vector dataParam, float cost)
    {
	super(parentFrame,"Bestellung", true);
	this.parentFrame = parentFrame;
	this.catalog = catalog;
	this.customer = customer;
	data = dataParam;
	this.cost = cost;
	GridBagLayout gridBagLayout = new GridBagLayout();
	GridBagConstraints gridBagConstraints = new GridBagConstraints();
	getContentPane().setLayout(gridBagLayout);
	setVisible(false);
	Label label1 = new Label("Bestellnummer: "+data.hashCode());
	gridBagConstraints.insets.top = 10;
	gridBagConstraints.insets.left = 10;
	gridBagConstraints.insets.right = 10;
	gridBagConstraints.gridx = 0;
	gridBagConstraints.gridy = 0;
	gridBagConstraints.gridwidth = 2;
	gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
	gridBagConstraints.weightx = 0;
	gridBagConstraints.fill = GridBagConstraints.NONE;
	gridBagLayout.setConstraints(label1, gridBagConstraints);
	getContentPane().setBackground(new Color(192,192,210));
	getContentPane().add(label1);

	JScrollPane tableAggregate = createTable(); // Tabelle erzeugen
	tableAggregate.setPreferredSize(new Dimension(tableAggregate.getPreferredSize().width,
	    (1+data.size())*(tableView.getRowHeight()+tableView.getRowMargin())+1));

	gridBagConstraints.gridx = 0;
	gridBagConstraints.gridy = 1;
	gridBagConstraints.gridwidth = 2;
	gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
	gridBagConstraints.weightx = 1;
	gridBagConstraints.weighty = 1;
	gridBagConstraints.fill = GridBagConstraints.BOTH;
	gridBagLayout.setConstraints(tableAggregate, gridBagConstraints);
	getContentPane().add(tableAggregate);

	JPanel infoPanel = new InfoPanel();
	gridBagConstraints.gridx = 0;
	gridBagConstraints.gridy = 2;
	gridBagConstraints.weightx = 0;
	gridBagConstraints.gridwidth = 1;
	gridBagConstraints.fill = GridBagConstraints.NONE;
	gridBagConstraints.weighty = 0;
	gridBagConstraints.anchor = GridBagConstraints.NORTHEAST;
	gridBagLayout.setConstraints(infoPanel, gridBagConstraints);
	infoPanel.setBackground(new Color(192,192,210));
	getContentPane().add(infoPanel);

	orderSumPanel = new OrderSumPanel();
	orderSumPanel.setCurrency(catalog.Currency);
	orderSumPanel.setTaxRate(catalog.Tax);
	orderSumPanel.setSum(cost);
	gridBagConstraints.gridx = 1;
	gridBagConstraints.gridy = 2;
	gridBagConstraints.weightx = 0;
	gridBagConstraints.fill = GridBagConstraints.NONE;
	gridBagConstraints.weighty = 0;
	gridBagConstraints.anchor = GridBagConstraints.NORTHEAST;
	gridBagLayout.setConstraints(orderSumPanel, gridBagConstraints);
	orderSumPanel.setBackground(new Color(192,192,210));
	getContentPane().add(orderSumPanel);

	ButtonPanel buttonPanel = new ButtonPanel();
	gridBagConstraints.insets.bottom = 20;
	gridBagConstraints.gridx = 0;
	gridBagConstraints.gridy = 3;
	gridBagConstraints.gridwidth = 2;
	gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
	gridBagConstraints.weightx = 1;
	gridBagConstraints.weighty = 0;
	gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
	gridBagLayout.setConstraints(buttonPanel, gridBagConstraints);
	buttonPanel.setBackground(new Color(192,192,210));
	getContentPane().add(buttonPanel);

	setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
	okButton.addActionListener(this);
	cancelButton.addActionListener(this);

	pack();
	setResizable(false);
	Dimension screenDimension = Toolkit.getDefaultToolkit().getScreenSize();
	Dimension ownDimension = getSize();
	int x = 0;
	int y = 0;
	x = (screenDimension.width - ownDimension.width) / 2;
	y = (screenDimension.height - ownDimension.height) / 2;
	setLocation(x, y);
    }

    /**
    * handles ActionListeners calls
    */
    public void actionPerformed(java.awt.event.ActionEvent event) {
	if (event.getSource() == okButton) {
	    String [] tmp = new String[dataModel.getRowCount()];
	    for (int row = 0; row < dataModel.getRowCount(); row++) {
		String rowString = new String("");
		for (int column = 0; column < dataModel.getColumnCount(); column++) {
		    rowString += dataModel.getValueAt(row, column);
		}
		tmp[row] = rowString;
	    }

	    Order tb = new Order(customer.name, tmp,
		Integer.toString(tmp.hashCode()), orderSumPanel.getSum());
	    dispose();
	    customer.sendOrder(parentFrame, catalog.Host, catalog.Port, tb);
	}
	dispose();
    }
}
