/*
 * catalog/awt/OrderPanel.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package catalog.awt;

import catalog.awt.OrderSumPanel;
import catalog.awt.ProductPanel;
import catalog.customer.Catalog;
import catalog.customer.Customer;
import catalog.zip.Zip;

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import java.text.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;
import javax.swing.event.*;

import smi.appConf.ApplicationConfiguration;
import smi.appConf.ActionConfiguration;

import ssonet.SSONETConstraints;

/**
* Catalogwindow with choice, productinfo,
* configuration, help and order
*/
public class OrderPanel extends JPanel implements ActionListener
{

    public static final short CATALOG = 1;
    public static final short ORDER = 2;
    public Customer customer;

    ProductTable tableView;

    JButton quitButton, orderButton;
    JLabel validity;

    Catalog catalog;
    int numberOfProducts;
    ProductPanel productPanel;
    OrderSumPanel orderSumPanel;

    private NumberFormat numberFormat;
    private boolean isModal = false;

    public TableModel dataModel;
    Vector data = new Vector();

    // Die berschriften
    final static String[] Headlines={
	"Name",
	"Nummer",
	"Anzahl",
	"Einheit",
	"Preis pro Einheit",
	"Gesamt"};
    class OrderTableModel extends AbstractTableModel {

        OrderTableModel () {
            data = new Vector();
        }

        public int getColumnCount() {
            return Headlines.length;
        }

        public int getRowCount() {
            // Hier mu die Anzahl der Zeilen zurueckgegeben werden
            return data.size(); // liefert Anzahl der Vektoren in data
        }

        public Object getValueAt(int row, int col) {
            // Hier mu das Object zurueckgegeben werden
            if (row<data.size()) {
                return ( ((Vector)(data.elementAt(row))).elementAt(col));
            } else {
                System.err.println("Fehler in OrderPanel.java: Zugriff auf nichtinitialisierte Tabellenzeile !\n");
                return (new String("Fehler"));
            }
        }

        /**
        * gibt den Spaltennamen zurueck
        */
        public String getColumnName(int column) {
            return Headlines[column];
        }

        /**
        * gibt zurueck, ob einzelnes Tabellenfeld editierbar ist oder nicht
        */
        public boolean isCellEditable(int row, int col) {
            return (col==2); // Nur die dritte Spalte soll editierbar sein, die Zaehlung ist nullbasiert
        }

        /**
        * Setzt einen neuen Wert innerhalb der Tabelle
        * Wenn row = -1, dann wird eine neue Zeile angehngt
        */
        public void setValueAt(Object aValue, int row, int column)
        {
            // setzt ein Datum, wird auch aufgerufen, wenn der User was aendern sollte
            if ((row== -1) || (row>=data.size())) {
                // neues Element hinzufuegen
                data.addElement(aValue);
            } else {
                // Element abaendern, insbesondere, wenn user was eingegeben hat
                // zuerst Vektor holen
                Vector temp_vector = (Vector) data.elementAt(row);
                temp_vector.setElementAt(aValue, column);
                // hat sich an der subtotal was gendert?
                if (column == 2) update(row);
            }
        }
    }


    class ButtonPanel extends Panel
    {
        public ButtonPanel() {
            GridBagLayout gridBagLayout = new GridBagLayout();
            GridBagConstraints gridBagConstraints = new GridBagConstraints();
            this.setLayout(gridBagLayout);
            gridBagConstraints.gridx = 0;
            gridBagConstraints.gridy = 0;
            gridBagConstraints.ipady = 2;
            gridBagConstraints.ipadx = 5;
            gridBagConstraints.weightx = 0.5;
            orderButton = new JButton("Bestellung ausfllen.");
            gridBagLayout.setConstraints(orderButton, gridBagConstraints);
            this.add(orderButton);

            quitButton = new JButton("            Ende            ");
            gridBagConstraints.gridx = 1;
            gridBagConstraints.gridy = 0;
            gridBagLayout.setConstraints(quitButton, gridBagConstraints);
            this.add(quitButton);
        }
    }


    /**
    * creates a catalogwindow for the customer
    */
    public OrderPanel(Customer customer)
    {
        this.customer = customer;

        setName("Bestellung");
        numberFormat = NumberFormat.getInstance();
        numberFormat.setMaximumFractionDigits(2);
        numberFormat.setMinimumFractionDigits(2);
        GridBagLayout gridBagLayout = new GridBagLayout();
        GridBagConstraints gridBagConstraints = new GridBagConstraints();
        this.setBackground(new Color(192,192,210));
        setLayout(gridBagLayout);
        validity = new JLabel();
        gridBagConstraints.insets.top = 5;
        gridBagConstraints.insets.left = 5;
        gridBagConstraints.insets.right = 5;
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        gridBagConstraints.weightx = 1;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagLayout.setConstraints(validity, gridBagConstraints);
        add(validity);

        JTextArea textField1 = new JTextArea("",0,0);
        productPanel = new ProductPanel(textField1);

        JScrollPane tableAggregate = createTable();
        tableAggregate.setPreferredSize(new Dimension(tableAggregate.getPreferredSize().width,150));
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.weightx = 1;
        gridBagConstraints.weighty = 1;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.insets.top = 5;
        gridBagLayout.setConstraints(tableAggregate, gridBagConstraints);
        add(tableAggregate);

        gridBagConstraints.insets.top = 5;
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.weightx = 1;
        gridBagConstraints.weighty = 0;
        gridBagLayout.setConstraints(productPanel, gridBagConstraints);
        productPanel.setBackground(new Color(192,192,210));
        add(productPanel);

        orderSumPanel = new OrderSumPanel();
        orderSumPanel.setBackground(new Color(192,192,210));
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.weightx = 0;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        gridBagConstraints.weighty = 0;
        gridBagConstraints.anchor = GridBagConstraints.NORTHEAST;
        gridBagLayout.setConstraints(orderSumPanel, gridBagConstraints);
        add(orderSumPanel);

        textField1.setEditable(false);
        textField1.setLineWrap(true);
        textField1.setWrapStyleWord(true);
        textField1.setBorder(new EtchedBorder(5));
        textField1.setRows(4);
        JScrollPane textFieldScroll = new JScrollPane(textField1);
        gridBagConstraints.weightx = 1;
        gridBagConstraints.weighty = 0;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.insets = new Insets(5,5,5,5);
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagLayout.setConstraints(textFieldScroll, gridBagConstraints);
        add(textFieldScroll);

        ButtonPanel buttonPanel = new ButtonPanel();
        buttonPanel.setBackground(new Color(192,192,210));
        gridBagConstraints.insets.bottom = 5;
        gridBagConstraints.insets.top = 10;
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.weightx = 1;
        gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = GridBagConstraints.NORTH;
        gridBagConstraints.weighty = 0;
        gridBagLayout.setConstraints(buttonPanel, gridBagConstraints);
        add(buttonPanel);

        orderButton.addActionListener(this);
        quitButton.addActionListener(this);
    }

    /**
    * Erzeugt die Liste
    */
    public float getOrderSum(Vector paramVector)
    {
        Vector  tempVector;
        String  tempString, newString;
        float   returnSum = 0;
        Float   tempFloat;

        tempString = new String();

        for (int i=0; i<paramVector.size(); i++) {
            newString ="";
            tempVector = (Vector) paramVector.elementAt(i);
            tempString = (String) tempVector.elementAt(5);
            try {
                returnSum += numberFormat.parse(tempString).floatValue();
            } catch (Exception e) {
                // Exceptions werden geworfen, wenn der Eintrag in Subtotal
                // leer ist.
            };
        }
        return returnSum;
    } // of getOrderSum()


      /**
      * Hier ist der Hauptteil der Deklaration der Tabelle enthalten
    */
    private JScrollPane createTable()
    {
        dataModel = new OrderTableModel();

        if (productPanel == null) System.err.println("Fehler: productPanel == NULL !!!");

        tableView = new ProductTable(dataModel, productPanel);
        tableView.setAutoResizeMode(tableView.AUTO_RESIZE_ALL_COLUMNS);

        tableView.setRowSelectionAllowed(true);        // Zeilenauswahl an
        tableView.setColumnSelectionAllowed(false);    // aber keine Spalten
        tableView.setCellSelectionEnabled(false);      // und keine einzelne Zelle
        tableView.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        tableView.getTableHeader().setReorderingAllowed(true);  // Spalten duerfen aber vertauscht werden

        DefaultTableCellRenderer renderer_one = new DefaultTableCellRenderer();
        renderer_one.setHorizontalAlignment(JLabel.RIGHT);
        tableView.getColumn(Headlines[4]).setCellRenderer(renderer_one);
        tableView.getColumn(Headlines[5]).setCellRenderer(renderer_one);
        tableView.getColumn(Headlines[3]).setCellRenderer(renderer_one);

        JScrollPane scrollpane = new JScrollPane(tableView);
        return scrollpane;
    } // End of createTable()




    /**
    * creates the menu at the upper edge of the window
    */
    private JMenuBar createMenu()
    {
        mainMenuBar = new JMenuBar();

        //menu1
        menu1 = new JMenu("Katalog");
        menu1.setMnemonic('k');

        loadCatalog = new JMenuItem("Laden");
        loadCatalog.setMnemonic('l');
        menu1.add(loadCatalog);

        requestCatalog = new JMenuItem("Anfordern");
        requestCatalog.setMnemonic('a');
        menu1.add(requestCatalog);

        menu1.addSeparator();

        exit = new JMenuItem("Ende");
        exit.setMnemonic('e');
        menu1.add(exit);
        mainMenuBar.add(menu1);

        //menu2
        menu2 = new JMenu("Einstellungen");
        menu2.setMnemonic('e');

        changeConfig = new JMenuItem("Katalog anfordern");
        changeConfig.setMnemonic('k');
        menu2.add(changeConfig);

        showConfig = new JMenuItem("Bestellung aufgeben");
        showConfig.setMnemonic('b');
        menu2.add(showConfig);
        mainMenuBar.add(menu2);

        menu3 = new JMenu("Hilfe");
        menu3.setMnemonic('h');

        about = new JMenuItem("Info");
        about.setMnemonic('i');
        menu3.add(about);
        mainMenuBar.add(menu3);

        exit.addActionListener(this);
        changeConfig.addActionListener(this);
        showConfig.addActionListener(this);
        loadCatalog.addActionListener(this);
        requestCatalog.addActionListener(this);
        about.addActionListener(this);
        return mainMenuBar;
    }

    /**
    * adds the menu to the upper edge of the window
    */
    public void addMenu(JApplet applet) {
        applet.setJMenuBar(createMenu());
    }

    /**
    * loads from Class
    * @see Catalog <br> all changeable values of the order
    */
    public boolean loadLocalCatalog()
    {
        Zip zip = new Zip();
        try {
            zip.open();
            catalog = zip.getCatalog();
            zip.close();

            numberOfProducts = catalog.numberOfRows;
            validity.setText(catalog.Info);
            orderSumPanel.setTaxRate(catalog.Tax);
            orderSumPanel.setCurrency(catalog.Currency);
            customer.frame.setTitle("Katalog von " + catalog.Title);

            String tmp, tr;
            int old_stringp=0, stringp=0;
            Vector zeile;
            float zuq;
            dataModel = new OrderTableModel();

            for (int y = 0; y < numberOfProducts; y++) {
                old_stringp=0;
                stringp = 0;
                tmp = catalog.temp[y];
                zeile = new Vector();
                for (int xy=0; xy<5; xy++) {
                    stringp = tmp.indexOf(";", old_stringp); // Zeigt auf Semikolon
                    if (xy!=4)
                        zeile.addElement(new String(tmp.substring(old_stringp, stringp)));
                    else {
                        tr = new String(tmp.substring(old_stringp, stringp));
                        zuq = numberFormat.parse(tr).floatValue();
                        tr = numberFormat.format(zuq)+" "+catalog.Currency;
                        zeile.addElement(tr);
                    }
                    old_stringp=stringp+1;
                }
                zeile.addElement(new String()); // Spalte fr Teilsumme
                dataModel.setValueAt((Object)zeile,-1,0);
            }
            orderSumPanel.setSum(getOrderSum(data));
        } catch (Exception e) {
            e.printStackTrace();
            JOptionPane.showMessageDialog(null,"Kein Katalog verfgbar. - Bitte neuen Katalog anfordern.",
		"Achtung", JOptionPane.INFORMATION_MESSAGE);
            return false;
        }

        return true;
    }

    JMenuBar mainMenuBar;
    JMenu menu1, menu2, menu3, menu4;
    JMenuItem changeConfig, showConfig, loadCatalog, requestCatalog, about, exit;

    /**
    * close the order window
    */
    void OrderPanelWindowClosing() {
        System.exit(0);
    }

    /**
    * shows the info window
    */
    public void about () {
        JOptionPane.showMessageDialog(null,
	    "Projekt SSONET Kataloganwendung",
	    "Information",
	    JOptionPane.INFORMATION_MESSAGE);
        isModal = false;
    }

    /**
    * handles ActionListener calls
    */
    public void actionPerformed(ActionEvent event)
    {
        Object object = event.getSource();
        if(isModal)
            return;
        isModal = true;
        if (object == changeConfig)
            changeConfigGetCatalog();
        else if (object == showConfig)
            changeConfigSendRequest();
        else if (object == quitButton || object == exit)
            System.exit(0);
        else if (object == about)
            about();
        else if (object == requestCatalog)
            requestCatalogAction(customer);
        else if (object == loadCatalog)
            loadCatalogAction(event);
        else if (object == orderButton)
            orderButtonAction();
        else if (object == tableView)
            System.out.println("ActionListener: tableView");
        else if (object == dataModel) {
            System.out.println("ActionListener: dataModel");
        }
    }

    /**
    * calls loadLocalCatalog() and displays an error message
    * when no catalog is available
    */
    void loadCatalogAction(java.awt.event.ActionEvent event) {
        System.out.print("\nloadCatalog");
        if (!loadLocalCatalog())
            JOptionPane.showMessageDialog(null,
            "Kein Katalog verfgbar. - Bitte neuen Katalog anfordern.",
            "Achtung",
            JOptionPane.INFORMATION_MESSAGE);
        isModal = false;
    }

    /**
    * creates a new MerchantListFrame for the customer
    */
    void requestCatalogAction(Customer customer) {
        MerchantListFrame merchantListFrame;
        merchantListFrame = new MerchantListFrame(customer, this);
        merchantListFrame.show();
        isModal = false;
    }

    /**
    * updates the subtotal price in column 5 and the total sum
    */
    public void update(int row) {
        int amount = 0;
        try {
            amount = numberFormat.parse(dataModel.getValueAt(row,2).toString()).intValue();
        } catch (ParseException e) {
            System.err.println("Konnte Anzahl nicht ermitteln,"+row);
        }
        if (amount == 0)
            //delete totalPrice
            dataModel.setValueAt(new String(new Integer(amount).toString()), row,5);
        else {
            String string = (String)dataModel.getValueAt(row,4);
            float price;
            try {
                price = numberFormat.parse(string).floatValue();
            } catch(Exception e) {
                price = 0;
            };
            float totalPrice = amount * price;
            numberFormat.setMinimumFractionDigits(2);
            numberFormat.setMaximumFractionDigits(2);

            if (totalPrice == 0)
                string = "";
            else
                string = numberFormat.format(totalPrice);

            dataModel.setValueAt(string, row,5);
        }
        orderSumPanel.setSum(getOrderSum(data));
    }

    /**
    * changes customers ActionConfiguration for catalog receiving
    */
    public void changeConfigGetCatalog() {
        System.out.println(SwingUtilities.windowForComponent(customer));
        SSONETConstraints configuration = new SSONETConstraints();
        configuration.changeMin (0,0,0,0);  // min should already 0,0,0,0
        configuration.changeDef (0,0,0,0);  // change defalut to 0,0,0,0
        configuration.changeMax (6,6,6,6);  // max should already 0,0,0,0
        ApplicationConfiguration.configureAction(this.customer.frame,
	    customer.name,
	    "Katalog anfordern",
	    CATALOG,
	    configuration);
        isModal = false;
    }

    /**
    * changes customers ActionConfiguration for ordering
    */
    public void changeConfigSendRequest() {
        SSONETConstraints configuration = new SSONETConstraints(); //defaults set
        ApplicationConfiguration.configureAction(this.customer.frame,
	    customer.name,
	    "Bestellung aufgeben",
	    ORDER,
	    configuration);
        isModal = false;
    }

    /**
    * An order will be issued.
    * Only products with an amount > 0 will be taken into consideration.
    * The orderform window will be opened
    */
    void orderButtonAction() {
        String productStrings[] = new String[numberOfProducts];
        int numberOfItems = 0;
        for (int i = 0; i < numberOfProducts; i++) {
            productStrings[i] = new String();
            //read amount
            String string = (String)dataModel.getValueAt(i,2);
            if (!string.equals("0") && !string.equals("")) {
                for (int j = 0; j < 6; j++)
                    productStrings[i] = productStrings[i] + (String)dataModel.getValueAt(i, j) + ";";
                numberOfItems++;
            }
        }
        if (numberOfItems == 0)
            JOptionPane.showMessageDialog(null,
            "Keine Produkte ausgewhlt",
            "Achtung",
            JOptionPane.INFORMATION_MESSAGE);
        else {
            String [] stringArray = new String[numberOfItems];
            numberOfItems = 0;
            for (int i = 0; i < numberOfProducts; i++) {
                if (!productStrings[i].equals("")) {
                    stringArray[numberOfItems] = new String(productStrings[i]);
                    numberOfItems++;
                }
            }
            Vector tempVector = (Vector)data.clone();
            String tempString;
            for (int index=0; index < tempVector.size(); index++) {
                tempString = (String)(((Vector)tempVector.elementAt(index)).elementAt(2));
                if (tempString.equals("") || tempString.equals("0")) {
                    tempVector.removeElementAt(index--);
                }
            }

            OrderForm orderForm = new OrderForm(customer.frame,
		customer,
		catalog,
		tempVector,
		getOrderSum(tempVector));
            orderForm.show();
        }
        isModal = false;
    }

    /**
    * Updates the customer window, as example when a new catalog is loaded
    */
    public void update () {
        tableView.update();
        customer.frame.validate();
    }
}
