/*
 * catalog/customer/Customer.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package catalog.customer;

import catalog.awt.AnimatedOrderDialog;
import catalog.awt.AnimatedCatalogDialog;
import catalog.awt.OrderForm;
import catalog.awt.OrderPanel;
import catalog.awt.PreloadWaitFrame;
import catalog.net.CatalogFile;
import catalog.net.Order;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.*;
import java.io.File;
import java.io.RandomAccessFile;

import javax.swing.*;
import javax.swing.border.TitledBorder;

import smi.io.IOStream;

import ssonet.SSONETConnection;
import ssonet.SSONETStream;
import ssonet.Preload;

/**
*   creates a Customer
*/
public class Customer extends JApplet {
    //    Preload preload = new Preload();
    public String name;
    static public OrderPanel orderPanel;
    public JFrame frame;

    /**
    * creates a customer with standard name
    */
    public Customer() {
	name = "Standardkunde";
    }

    /**
    * creates a customer with name n
    */
    public Customer(String n) {
	name = n;
    }

    /**
    * creates the order frame, adds the menu and loads the local catalog
    */
    public void init() {
	orderPanel = new OrderPanel(this);
	getContentPane().add(orderPanel);
	orderPanel.addMenu(this);
	orderPanel.loadLocalCatalog();
    }

    /**
    * Sends an order to the merchanthost at the merchantHostPort. The parentFrame
    * is necessary for the modal representation of the AnimatedOrderDialog.
    * Returns true if action was successful, otherwise false.
    */
    public boolean sendOrder(JFrame parentFrame, String merchantHost, int merchantHostPort, Order order) {
	try {
	    AnimatedOrderDialog animatedOrderDialog = new AnimatedOrderDialog(parentFrame,
		this,
		merchantHost,
		merchantHostPort,
		order);
	    return true;
	} catch(Exception e) {
	    JOptionPane.showMessageDialog(parentFrame,
		"Interner Fehler bei der Bestellung",
		"Fehler", JOptionPane.ERROR_MESSAGE);
	    System.out.println("Fehler beim Bestellen:\n"+e);
	    return false;
	}
    }


    /**
    * Sends a catalog order to the merchanthost at the merchantHostPort. The parentFrame
    * is necessary for the modal representation of the AnimatedOrderDialog.
    * Returns true if action was successful, otherwise false.
    */
    public boolean requestCatalog(JFrame parentFrame, String merchantName, String merchantHost, int merchantHostPort) {
	try {
	    AnimatedCatalogDialog animatedCatalogDialog = new AnimatedCatalogDialog(parentFrame,
		this,
		merchantHost,
		merchantHostPort);
	    frame.validate();
	    return true;
	} catch(Exception e) {
	    System.out.println("Fehler beim Dialog erstellen.");
	    return false;
	}
    }

    /**
    * Writes the catalogFile into the local catalog "catalog.zip" in the directory ./data
    * After that the local catalog will be loaded into the OrderPanel.
    * Returns true if action was successful, otherwise false.
    */
    public boolean getCatalog(CatalogFile catalogFile) {
	String fileName = System.getProperty("ssonet_cd")+"catalog/cfg/katalog.zip";
	try {
	    File file = new File(fileName);
	    if (file.exists())
		file.delete();
	    RandomAccessFile randomAccessFile = new RandomAccessFile(file,"rw");
	    randomAccessFile.seek(0);
	    randomAccessFile.write(catalogFile.allBytes);
	    randomAccessFile.close();
	    System.out.println("Katalog erfolgreich gespeichert: " + fileName);
	    orderPanel.loadLocalCatalog();
	    orderPanel.update();
	    return true;
	} catch (Exception e) {
	    e.printStackTrace();
	    return false;
	}
    }

    public Dimension getSize() {
	return frame.getSize();
    }

    /**
    * main customer procedure
    * Creates a JFrame and a customer with the name supplied. If no name was
    * supplied, the name "Standard customer" will be used. The created customer
    * will be assigned to a JFrame. The customer will be initialized and the JFrame
    * will be packed. That means, that the components of the JFrame will be displayed
    * in their preferred size. Last but not least the JFrame will be displayed.
    *
    * System-Property "ssonet_userdir" is set to the username.
    */
    public static void main(String[] argv) {
	String customerName;
	if (argv.length == 0)
	    customerName = "Standardkunde";
	else
	    customerName = new String(argv[0]);
	System.out.println("Kundenname:" + customerName);
	System.setProperty("ssonet_userdir",customerName);
	PreloadWaitFrame preloadWaitFrame = new PreloadWaitFrame ();
	Customer customer = new Customer(customerName);
	customer.frame = new JFrame();
	customer.frame.setContentPane(customer);
	customer.frame.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
	customer.frame.addWindowListener(new WindowAdapter()
	{
	    public void windowClosing(java.awt.event.WindowEvent event)
	    {
		System.exit(0);
	    }
	});
	customer.init();
	customer.frame.pack();
	Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	customer.frame.setSize(customer.frame.getSize().width,
				java.lang.Math.min(screenSize.height-50, customer.getSize().height));
	Dimension ownDimension = customer.frame.getSize();
	customer.frame.setLocation((screenSize.width - ownDimension.width) / 2,
	    (screenSize.height - ownDimension.height) / 2);
	customer.frame.show();
	// Testen, ob Schutzzieleinstellungen schon vorhanden sind
	if (smi.appConf.ApplicationConfiguration.getActionConfiguration(customerName, orderPanel.ORDER) == null) {
	    System.out.println("Bestelleinstellung (Order) nicht vorhanden.");
	    orderPanel.changeConfigSendRequest();
	}
	if (smi.appConf.ApplicationConfiguration.getActionConfiguration(customerName, orderPanel.CATALOG) == null) {
	    System.out.println("Katalogeinstellung (Order Catalog) nicht vorhanden.");
	    orderPanel.changeConfigGetCatalog();
	}
	customer.frame.requestFocus();
    }
}
