/*
 * catalog/merchant/Merchant.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package catalog.merchant;

import catalog.merchant.MerchantServer;
import catalog.net.Order;

import java.awt.*;

//import javax.swing.JFrame;
import javax.swing.*;

import smi.appConf.ApplicationConfiguration;
import smi.appConf.ActionConfiguration;
import ssonet.Preload;
import ssonet.SSONETConstraints;

/**
* create a merchant
*/
public class Merchant extends JFrame {
    Preload preload = new Preload();
    public final static short CATALOG = 1;
    public final static short ORDER  = 2;

    MerchantServer merchantServer;
    MenuItem sendMenuItem;
    MenuItem orderMenuItem;
    MenuItem exitMenuItem;
    MenuItem restartMenuItem;

    private static Merchant merchant;
    private static String merchantName;
    private String catalogFileName;
    private int portNumber;

    /**
    * sets the icon, creates the menu
    */
    private void createDialog() {
	setIconImage(new ImageIcon(getClass().getResource("images/icon.gif")).getImage());
	MenuBar sendCatalog = new MenuBar();
	Menu mdatei = new Menu("Datei");
	restartMenuItem = new MenuItem("Neustart");
	mdatei.add(restartMenuItem);
	exitMenuItem = new MenuItem("Ende");
	mdatei.add(exitMenuItem);
	sendCatalog.add(mdatei);
	Menu mconfig = new Menu("Einstellung");
	Menu menuTrans = new Menu("Verbindung");
	mconfig.add(menuTrans);

	sendMenuItem = new MenuItem("Katalog versenden");
	menuTrans.add(sendMenuItem);
	orderMenuItem = new MenuItem("Bestellung empfangen");
	menuTrans.add(orderMenuItem);
	sendCatalog.add(mconfig);
	setMenuBar(sendCatalog);

	getContentPane().add(new Label("Versandhaus " + merchantName
					+ ": Onlineverkauf"));

	SymWindow aSymWindow = new SymWindow();
	this.addWindowListener(aSymWindow);
	SymAction lSymAction = new SymAction();
	exitMenuItem.addActionListener(lSymAction);
	restartMenuItem.addActionListener(lSymAction);
	sendMenuItem.addActionListener(lSymAction);
	orderMenuItem.addActionListener(lSymAction);
    }

    /**
    * creates a merchant and a merchantServer
    */
    public Merchant(String merchantName, String catalogFileName, int portNumber)  {
	this.merchantName = merchantName;
	this.catalogFileName = catalogFileName;
	this.portNumber = portNumber;
	createDialog();
	setResizable(false);
	setTitle(merchantName);
	merchantServer = new MerchantServer(this, merchantName, catalogFileName, portNumber);
	pack();
	show();
    }

    /**
    * Symantec Window
    */
    class SymWindow extends java.awt.event.WindowAdapter {
	public void windowClosing(java.awt.event.WindowEvent event) {
	    WindowClosing();
	}
    }

    /**
    * closes a SymWindow
    */
    void WindowClosing() {
	//hide the Frame
	setVisible(false);
	System.exit(0);
    }

    /**
    * Symantec ActionListener
    */
    class SymAction implements java.awt.event.ActionListener {
	public void actionPerformed(java.awt.event.ActionEvent event) {
	    Object object = event.getSource();
	    if (object == exitMenuItem)
		WindowClosing();
	    else if (object == sendMenuItem)
		changeConfigSendCatalog();
	    else if (object == orderMenuItem)
		changeConfigGetRequest();
	    else if (object == restartMenuItem)
		restartMerchantServer();
	}
    }

    /**
    * restarts the MerchantServer
    */
    void restartMerchantServer() {
	merchantServer.stopServer();
	System.gc();
	merchantServer = new MerchantServer(this, merchantName, catalogFileName, portNumber);
    }

    /**
    * change ActionConfiguration for catalog sending
    */
    static void changeConfigSendCatalog() {
	SSONETConstraints configuration = new SSONETConstraints();
	ApplicationConfiguration.configureAction(merchant,
	    merchantName,
	    "Aktionskonfiguration: Katalog versenden",
	    CATALOG,
	    configuration);
    }

    /**
    * change ActionConfiguration for receiving order requests
    */
    static void changeConfigGetRequest() {
	SSONETConstraints configuration = new SSONETConstraints();
	ApplicationConfiguration.configureAction(merchant,
	    merchantName,
	    "Aktionskonfiguration: Bestellung empfangen",
	    ORDER,
	    configuration);
    }

    /**
    * merchants main procedure
    * Bentigte Umgebungsvariablen:
    *   ssonet_cd : Root-Verzeichnis der SSONET-Architektur
    *   ssonet_cfg: Konfigurationsverzeichnis der SSONET-Architektur
    *   ssonet_help: Verzeichnis der Hilfetexte
    *   Die Variable "ssonet_user" wird auf den Applikationsnamen gesetzt.
    */
    public static void main(String[] argv) {
	String fileName = System.getProperty("ssonet_cd") + "catalog/cfg/merchant/Katalog_blitz.zip";
	merchantName = "Blitz";
	int portNumber = 2001;
	try {
	    switch (argv.length) {
	    case 3:
		// Port des Hndlers
		portNumber = (new Integer(argv[2])).intValue();
	    case 2:
		// Dateiname des Katalogs
		fileName = new String(argv[1]);
	    case 1:
		// Name des Hndlers
		merchantName = new String(argv[0]);
		break;
	    default:
		System.out.println("Standardeinstellung (\"Blitz\", Katalog:\"Katalog_blitz.zip\", Port:2001) wird verwendet!");
		System.out.println("Aufruf: java Merchant [Merchantname] [Catalogname] [Portnumber]\n");
	    }
	    System.setProperty("ssonet_userdir",merchantName);
	    merchant = new Merchant(merchantName, fileName, portNumber);
	}
	catch (Exception e) {
	    System.out.println("\nFehler in Merchantmain:"+e);
	}
	// Test, ob Schutzzielkonfigurationseinstellungen gefunden werden
	if (smi.appConf.ApplicationConfiguration.getActionConfiguration(merchantName, ORDER) == null) {
	    System.out.println("Keine Bestelleinstellung (Order) vorhanden.");
	    changeConfigGetRequest();
	}
	if (smi.appConf.ApplicationConfiguration.getActionConfiguration(merchantName, CATALOG) == null) {
	    System.out.println("Keine Katalogeinstellung (request Catalog) vorhanden.");
	    changeConfigSendCatalog();
	}
    }
}
