/*
 * gateway/GServer.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package gateway;

import javax.swing.JFrame;
import java.net.*;
import java.io.*;
import java.util.Hashtable;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import xjava.security.Cipher;
import java.security.PublicKey;
import java.security.PrivateKey;

import smi.appConf.ApplicationConfiguration;
import smi.io.*;

import ssonet.SSONETConnection;
import ssonet.SSONETServerConnection;
import ssonet.SSONETConstants;
import ssonet.Preload;
import ssonet.SSONETConstraints;
import ssonet.KeyExchange;
import smi.baseConf.BaseConfiguration;
import sun.security.x509.X509Cert;
import smi.mechanisms.confidentiality.RSA;
import java.security.Key;
import smi.io.IOStream;
import smi.mechanisms.*;
public class GServer extends JFrame implements Serializable {
    final static int PORT = 5000;
    Preload p = new Preload();
    IOStream rsaStream, ioStream;
    ServerSocket serversocket = null;
    Socket socket = null;
    // Action-ID
    Cipher CipherInH, CipherOutH, CipherInB, CipherOutB;
    static String KEY_MODE_PREFERENCELIST = "Mode.Preferencelist";
    String mechanismName;

    protected X509Cert ownTestCertificate, ownCipherCertificate;
    private X509Cert mechanismCertificate;
    protected PublicKey ownPublicKey, trustCenterTestKey;

    private String packageName;
    private static PrivateKey ownPrivateKey;
    boolean isServer=true;
    KeyExchange GWKeyExchange;

    static public void main(String[] argv) {
	new GServer();
    }

    private GServer() {
	try {
	    serversocket = new ServerSocket(PORT);
	} catch(Exception e) {
	    System.out.println("Error creating Socket!");
	    e.printStackTrace();
	    System.exit(1);
	}

	while(true) {
	    try {
		try {
		    //            serversocket = new ServerSocket(PORT);
		    socket = serversocket.accept();
		} catch (UnknownHostException e) {
		    System.err.println(e);
		} catch (IOException e) {
		    System.err.println(e);
		}
		System.out.println("Server Socket created");
		InputStream in = socket.getInputStream();
		OutputStream out = socket.getOutputStream();
		IOStream ioStream = new ObjectIOStream(in, out);
		try {
		    initConnection();
		    GWKeyExchange = new KeyExchange(ioStream,
			null,
			ownTestCertificate,
			ownCipherCertificate,
			trustCenterTestKey,
			isServer);

		    mechanismCertificate = GWKeyExchange.readCertificate(System.getProperty("ssonet_cfg","")
			+"users/"+System.getProperty("ssonet_userdir")+"/confidentiality/RSACertificate.der");
		    PublicKey publicKey = GWKeyExchange.getPartnerMechanismKey(mechanismCertificate);
		    System.out.println("publicKey: "+publicKey);
		    ownPrivateKey = GWKeyExchange.readRSAPrivateKey(System.getProperty("ssonet_cfg","")
			+"users/"+System.getProperty("ssonet_userdir")+"/KeyExchangeRSAPrivateKey.der");
		    System.out.println("ownPrivateKey: "+ownPrivateKey);

		    MechanismConfiguration mechs = (MechanismConfiguration)ioStream.readObject();
		    String mechanismNameBack = (String)ioStream.readObject();
		    System.out.println("mechanismNameBack: "+mechanismNameBack);
		    packageName = (String)ioStream.readObject();
		    Key sessionKeyHin = (Key)ioStream.readObject();
		    String mechanismNameHin = sessionKeyHin.getAlgorithm();

		    /*      System.out.println("read now decrypted Objects.");
		    byte[] bobj = (byte[])ioStream.readObject();
		    bobj = decrypt(bobj, ownPrivateKey);
		    System.out.println("decrypt ready.");
		    ByteArrayInputStream bais = new ByteArrayInputStream(bobj);
		    ObjectInputStream ois = new ObjectInputStream(bais);
		    Key sessionKeyHin = (Key)ois.readObject();
		    */

		    //hole sessionKey Rckmechanismus vom Gateway
		    System.out.println("Hole jetzt sessionKey fuer BackMech");
		    Key sessionKeyBack = GWKeyExchange.getSessionKey(mechanismNameBack);
		    System.out.println("Hole jetzt sessionKey ...ready");

		    MechanismConfiguration backMech = (MechanismConfiguration)mechs.get("BACK");
		    MechanismConfiguration hinMech  = (MechanismConfiguration)mechs.get("HIN");
		    System.out.println("HMN: "+sessionKeyHin.getAlgorithm());
		    System.out.println("BMN: "+sessionKeyBack.getAlgorithm());

		    String mode = ((String[])backMech.get(KEY_MODE_PREFERENCELIST))[0];
		    initCipherBack(mechanismNameBack, sessionKeyBack, mode);
		    mode = ((String[])hinMech.get(KEY_MODE_PREFERENCELIST))[0];
		    initCipherHin(mechanismNameHin, sessionKeyHin, mode);

		    while(ioStream.available()==0) {
			//immer vom Client
			byte[] inp  = (byte[])ioStream.readObject();

			// entschlsseln!
			inp = CipherInB.crypt(inp);

			Hashtable ht  = (Hashtable)(new ObjectInputStream(new ByteArrayInputStream(inp))).readObject();
			System.out.println("Object empfangen.");
			Object object = ht.get("OBJECT");

			if(packageName.equals("confidentiality.")) {
			    System.out.println("object is of type "+object.getClass().getName());
			    // "byte[] b = (byte[])object;"
			    byte[] b = null;
			    {
				ByteArrayOutputStream bo = new ByteArrayOutputStream();
				ObjectOutputStream oo = new ObjectOutputStream(bo);
				oo.writeObject(object);
				b = bo.toByteArray();
			    }
			}

			if(((String)ht.get("TODO")).equals("ENCRYPT")) {
			    System.out.println("Object mit Hinmech. verschlsseln.");
			    //Nachricht vom Client
			    //muss vom Gateway verschlsselt werden (HinMech).
			    System.out.println(packageName);
			    if(packageName.equals("confidentiality.")) {
				System.out.println(packageName);
				ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
				ObjectOutputStream objectOutputStream = new ObjectOutputStream(byteArrayOutputStream);
				objectOutputStream.writeObject(object);
				objectOutputStream.flush();
				byte[] buffer = (byte[])byteArrayOutputStream.toByteArray();
				buffer = CipherOutH.crypt(buffer);
				ioStream.writeObject(buffer);
				objectOutputStream.close();
				byteArrayOutputStream.close();
				System.out.println("Object mit Hinmech. verschlsselt.");
			    } else if(packageName.equals("integrity.")) {
				System.out.println(packageName);
				SignatureObject signatureObject = new SignatureObject(object);
				System.out.println(" sigObj.: "+signatureObject);
				byte[] MAC = new byte[CipherOutH.blockSize()];
				byte[] buffer = CipherOutH.crypt(signatureObject.getObjectAsByteArray());
				System.arraycopy(buffer, buffer.length-MAC.length, MAC,0, MAC.length);
				signatureObject.setSignature(MAC);
				ioStream.writeObject(signatureObject);
				System.out.println("Object mit Hinmech. verschlsselt.");
			    }
			} else if(((String)ht.get("TODO")).equals("DECRYPT&ENCRYPT")) {
			    System.out.println("Object umcodieren.");
			    if(packageName.equals("confidentiality.")) {
				//Object umcodieren  (entschl.(HinMech), verschl.(RckMech))
				byte[] buffer = (byte[])object;
				buffer = CipherInH.crypt(buffer);

				//System.out.println("Zwischerg2.: "+ new String(buffer)+" lgt.: "+buffer.length);
				int len = new String(buffer).length();
				if (len>50)
				    len = 50;
				System.out.println("Zwischerg2.: "+ new String(buffer, 0, len)+" lgt.: "+buffer.length);

				Object returnObject = (new ObjectInputStream(new ByteArrayInputStream(buffer))).readObject();
				System.out.println("GServer - Classname:" + (String)(returnObject.getClass().getName()));

				//verschlsseln
				buffer = CipherOutB.crypt(buffer);

				ioStream.writeObject(buffer);
				System.out.println("Object umcodiert.");
			    } else if(packageName.equals("integrity.")) {
				SignatureObject signatureObject = (SignatureObject)object;
				System.out.println("read1 sigObj.: "+signatureObject);
				byte[] MAC = new byte[CipherOutH.blockSize()];
				byte[] buffer = CipherInH.crypt(signatureObject.getObjectAsByteArray());
				System.arraycopy(buffer, buffer.length-MAC.length, MAC,0, MAC.length);
				byte[] verify = signatureObject.getSignature();
				if(MAC.length == verify.length) {
				    for(int i = 0; i<MAC.length; i++)
					if(MAC[i] != verify[i]) throw new Exception();
				    System.out.println("read2 sigObj.: "+signatureObject);
				    object = (Object)signatureObject.getObject();
				}
				System.out.println("second part");
				signatureObject = new SignatureObject(object);
				MAC = new byte[CipherOutB.blockSize()];
				buffer = CipherOutB.crypt(signatureObject.getObjectAsByteArray());
				System.arraycopy(buffer, buffer.length-MAC.length, MAC,0, MAC.length);
				signatureObject.setSignature(MAC);
				ioStream.writeObject(signatureObject);
				System.out.println("Object umcodiert.");
			    }
			}
		    }
		    out.close();
		    in.close();
		    ioStream.close();
		    socket.close();
		    System.out.println("Socket closed");
		} catch(IOException e) {
		    System.out.println("inner catch");
		    out.close();
		    in.close();
		    ioStream.close();
		    socket.close();
		    System.out.println("Socket closed");
		}
	    } catch(Exception e) {
		System.out.println("outer catch");
		e.printStackTrace();
	    }
	}
    }
    //Hinmechanismus initialisieren
    private void initCipherHin(String mechName, Key sessionKey, String mode) {
	try {
	    CipherInH =  Cipher.getInstance(mechName + "/" + mode + "/OneAndZeroes");
	    CipherOutH = Cipher.getInstance(mechName + "/" + mode + "/OneAndZeroes");
	    CipherInH.initDecrypt(sessionKey);
	    CipherOutH.initEncrypt(sessionKey);
	    System.out.println(mechName + " instance initialized (Hinmech.)");
	} catch(Exception e) {
	    System.out.println(e);
	}
    }
    //Rckmechanismus initialisieren
    private void initCipherBack(String mechName, Key sessionKey, String mode) {
	try {
	    CipherInB =  Cipher.getInstance(mechName + "/" + mode + "/OneAndZeroes");
	    CipherOutB = Cipher.getInstance(mechName + "/" + mode + "/OneAndZeroes");
	    CipherInB.initDecrypt(sessionKey);
	    CipherOutB.initEncrypt(sessionKey);
	    System.out.println(mechName + " instance initialized (Rueckmech.)");
	} catch(Exception e) {
	    System.out.println(e);
	}
    }

    private void initConnection() {
	System.out.println("reading trustcenter certificate:");
	X509Cert trustCenterCertificate = KeyExchange.readCertificate(
	    System.getProperty("ssonet_cfg","")+"trustcenter/TrustCenterCertificate.der");
	trustCenterTestKey = trustCenterCertificate.getPublicKey();

	System.out.println("testing trustcenter certificate.");
	if (!KeyExchange.testCertificate(trustCenterCertificate, trustCenterTestKey))
	    throw new SecurityException();

	System.out.println("reading ownTestCertificate:");
	ownTestCertificate = KeyExchange.readCertificate(
	    System.getProperty("ssonet_cfg","")
		+"users/"+System.getProperty("ssonet_userdir")+"/KeyExchangeDSACertificate.der");
	ownPublicKey = ownTestCertificate.getPublicKey();

	System.out.println("testing ownTestCertificate.");
	if (!KeyExchange.testCertificate(ownTestCertificate, trustCenterTestKey))
	    throw new SecurityException();

	System.out.println("reading ownCipherCertificate:");
	ownCipherCertificate = KeyExchange.readCertificate(
	    System.getProperty("ssonet_cfg","")
		+"users/"+System.getProperty("ssonet_userdir")+"/KeyExchangeRSACertificate.der");

	System.out.println("testing ownCipherCertificate.");
	if (!KeyExchange.testCertificate(ownCipherCertificate, ownPublicKey))
	    throw new SecurityException();
    }
    /** decrypt ByteArray with private key
    * @param input data to decrypt
    * @param privateKey private key
    * @return byte[] decrypted data
    */
    private byte[] decrypt(byte[] input, PrivateKey privateKey) {
	try {
	    Cipher cipherIn=Cipher.getInstance("RSA/ECB/OneAndZeroes");
	    cipherIn.initDecrypt(privateKey);
	    byte[] output=cipherIn.crypt(input);
	    return output;
	} catch(java.lang.Exception e) {
	    System.out.println("Error in decrypt");
	    e.printStackTrace();
	}
	return null;
    }
    public byte[] getObjectAsByteArray(Object object) {
	try {
	    ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
	    ObjectOutputStream objectOutputStream = new ObjectOutputStream(byteArrayOutputStream);
	    objectOutputStream.writeObject(object);
	    objectOutputStream.flush();
	    byte[] b = byteArrayOutputStream.toByteArray();
	    System.out.println("getObjectAsByteArray lgt.: "+b.length);
	    return b;
	} catch (Exception e) {
	    System.out.println("Error while transforming into ByteArray.");
	    e.printStackTrace();
	    return null;
	}
    }
}
