/*
 * goals/awt/UserSecurityDefinitionDialog.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package goals.awt;

import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.URL;
import java.util.Enumeration;

import help.Help;
import javax.swing.ButtonGroup;
import javax.swing.JFrame;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTabbedPane;
import javax.swing.border.LineBorder;
import javax.swing.border.TitledBorder;
import javax.swing.tree.DefaultMutableTreeNode;

import goals.SecuritySettings;

public class UserSecurityDefinitionDialog extends JDialog implements ActionListener {
    private final static int CHANGE_TRANSACTION = 0;
    private final static int CHANGE_ACTION = 1;
    private final static int ABORT = 2;

    private Container container;
    private JButton okButton,
	cancelButton,
	helpButton,
	resetHighPrivacyButton,
	resetMiddlePrivacyButton,
	resetLowPrivacyButton,
	resetHighCorrectnessButton,
	resetMiddleCorrectnessButton,
	resetLowCorrectnessButton;
    private JPanel privacyPanel, correctnessPanel;
    private ButtonGroupComponent privacyContentCipherButtons,
	privacyContentHideButtons,
	privacyIWantAnonymityButtons,
	privacyIWantInvisibilityButtons,
	privacyPartnerShouldAnonymityButtons,
	correctnessContentIntegrityButtons,
	correctnessContentAvailabilityButtons,
	correctnessIWantAccountabilityButtons,
	correctnessIWantReachabilityButtons,
	correctnessPartnerShouldAccountabilityButtons,
	correctnessPartnerShouldReachabilityButtons;
    private DefaultMutableTreeNode selectedNode;
    private int[] originalValues;
    private JTabbedPane tabbedPane;
    private boolean makePreviousOwnPrivacyChanges,
	makePreviousPartnerPrivacyChanges,
	makeNextOwnPrivacyChanges,
	makeNextPartnerPrivacyChanges,
	changeOwnTransaction,
	changePartnerTransaction;
    private Help help;
    private CommunicationSecurityDialog communicationSecurityDialog;
    private WindowAdapter myWindowAdapter = new WindowAdapter() {
	public void windowClosing(WindowEvent e) {
	    dispose();
	}
    };

    public UserSecurityDefinitionDialog(CommunicationSecurityDialog communicationSecurityDialog,
	DefaultMutableTreeNode selectedNode,
	Help help) {
	super(new Frame(),"Benutzerdefinierte Sicherheitsforderungen - " + selectedNode.toString(), true);

	this.communicationSecurityDialog = communicationSecurityDialog;
	setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
	addWindowListener(myWindowAdapter);

	this.selectedNode = selectedNode;
	this.help = help;

	originalValues = ((SecuritySettings)selectedNode.getUserObject()).getUserDefinedValues();

	GridBagLayout gridBagLayout = new GridBagLayout();
	GridBagConstraints gridBagConstraints = new GridBagConstraints();

	// pre-defined constraints
	GridBagConstraints singleLabelConstraints = new GridBagConstraints();
	singleLabelConstraints.anchor = GridBagConstraints.NORTHWEST;
	singleLabelConstraints.insets = new Insets(0,5,0,5);
	singleLabelConstraints.gridwidth = GridBagConstraints.REMAINDER;

	GridBagConstraints firstLabelConstraints = new GridBagConstraints();
	firstLabelConstraints.anchor = GridBagConstraints.NORTHWEST;
	firstLabelConstraints.insets = new Insets(20,5,0,5);
	firstLabelConstraints.gridwidth = GridBagConstraints.REMAINDER;

	GridBagConstraints secondLabelConstraints = new GridBagConstraints();
	secondLabelConstraints.anchor = GridBagConstraints.NORTHWEST;
	secondLabelConstraints.insets = new Insets(0,5,0,5);
	secondLabelConstraints.gridwidth = GridBagConstraints.REMAINDER;

	GridBagConstraints panelConstraints = new GridBagConstraints();
	panelConstraints.anchor = GridBagConstraints.NORTHWEST;
	panelConstraints.fill = GridBagConstraints.BOTH;
	panelConstraints.weightx = 1;
	panelConstraints.weighty = 1;

	GridBagConstraints lastPanelConstraints = new GridBagConstraints();
	lastPanelConstraints.fill = GridBagConstraints.BOTH;
	lastPanelConstraints.gridwidth = GridBagConstraints.REMAINDER;
	lastPanelConstraints.weightx = 1;
	lastPanelConstraints.weighty = 1;

	GridBagConstraints buttonGroupConstraints = new GridBagConstraints();
	buttonGroupConstraints.anchor = GridBagConstraints.NORTHWEST;
	buttonGroupConstraints.gridwidth = GridBagConstraints.REMAINDER;
	buttonGroupConstraints.weightx = 1;
	buttonGroupConstraints.weighty = 1;

	GridBagConstraints resetPanelConstraints = new GridBagConstraints();
	resetPanelConstraints.gridwidth = GridBagConstraints.REMAINDER;
	resetPanelConstraints.fill = GridBagConstraints.HORIZONTAL;
	resetPanelConstraints.weightx = 1;

	GridBagConstraints leftButtonConstraints = new GridBagConstraints();
	leftButtonConstraints.anchor = GridBagConstraints.EAST;
	leftButtonConstraints.insets = new Insets(0,0,7,0);
	leftButtonConstraints.weightx = 1;

	GridBagConstraints middleButtonConstraints = new GridBagConstraints();
	middleButtonConstraints.insets = new Insets(0,7,7,0);

	GridBagConstraints rightButtonConstraints = new GridBagConstraints();
	rightButtonConstraints.anchor = GridBagConstraints.WEST;
	rightButtonConstraints.insets = new Insets(0,7,7,0);
	rightButtonConstraints.weightx = 1;

	container = getContentPane();
	container.setLayout(gridBagLayout);

	JLabel label = new JLabel("Forderungen einstellen fr:");
	gridBagConstraints.anchor = GridBagConstraints.WEST;
	gridBagConstraints.insets = new Insets(5,5,5,5);
	gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER;
	gridBagLayout.setConstraints(label, gridBagConstraints);
	container.add(label);

	privacyPanel = new JPanel();
	privacyPanel.setLayout(gridBagLayout);

	LineBorder lineBorder = new LineBorder(Color.black);
	TitledBorder titledBorder = new TitledBorder(lineBorder);
	titledBorder.setTitle("Inhalte");

	JPanel panel = new JPanel();
	gridBagLayout.setConstraints(panel, panelConstraints);
	panel.setBorder(titledBorder);
	panel.setLayout(gridBagLayout);
	privacyPanel.add(panel);

	label = new JLabel("vertraulich kommunizieren");
	label.setForeground(Color.black);
	gridBagLayout.setConstraints(label, singleLabelConstraints);
	panel.add(label);

	privacyContentCipherButtons = new ButtonGroupComponent("images/confidentialityOn.gif",
	    "images/confidentialityEqual.gif",
	    "images/confidentialityOff.gif",
	    true,
	    "Nachrichteninhalte sollen nur dem Partner bekannt werden",
	    "Nachrichteninhalte drfen allen bekannt werden");
	privacyContentCipherButtons.addActionListener(this);
	gridBagLayout.setConstraints(privacyContentCipherButtons, buttonGroupConstraints);
	panel.add(privacyContentCipherButtons);

	label = new JLabel(" ");
	label.setForeground(Color.black);
	gridBagLayout.setConstraints(label, firstLabelConstraints);
	panel.add(label);

	label = new JLabel("verdeckt kommunizieren");
	label.setForeground(Color.black);
	gridBagLayout.setConstraints(label, secondLabelConstraints);
	panel.add(label);

	privacyContentHideButtons = new ButtonGroupComponent("images/hiddenOn.gif",
	    "images/hiddenEqual.gif",
	    "images/hiddenOff.gif",
	    true,
	    "Von der geheimen Nachricht darf nur der Partner wissen",
	    "Von der geheimen Nachricht drfen alle wissen");
	privacyContentHideButtons.addActionListener(this);
	gridBagLayout.setConstraints(privacyContentHideButtons, buttonGroupConstraints);
	panel.add(privacyContentHideButtons);

	panel = new JPanel();
	gridBagLayout.setConstraints(panel, panelConstraints);
	privacyPanel.add(panel);

	titledBorder = new TitledBorder(lineBorder);
	titledBorder.setTitle("Ich will");
	panel.setBorder(titledBorder);
	panel.setLayout(gridBagLayout);

	label = new JLabel("anonym kommunizieren");
	label.setForeground(Color.black);
	gridBagLayout.setConstraints(label, singleLabelConstraints);
	panel.add(label);

	privacyIWantAnonymityButtons = new ButtonGroupComponent("images/anonymityOn.gif",
	    "images/anonymityEqual.gif",
	    "images/anonymityOff.gif",
	    true,
	    "Ich will anonym sein",
	    "Andere sollen meine Identitt erfahren knnen");
	privacyIWantAnonymityButtons.addActionListener(this);
	gridBagLayout.setConstraints(privacyIWantAnonymityButtons, buttonGroupConstraints);
	panel.add(privacyIWantAnonymityButtons);

	label = new JLabel(" ");
	label.setForeground(Color.black);
	gridBagLayout.setConstraints(label, firstLabelConstraints);
	panel.add(label);

	label = new JLabel("unbeobachtbar kommunizieren");
	label.setForeground(Color.black);
	gridBagLayout.setConstraints(label, secondLabelConstraints);
	panel.add(label);

	privacyIWantInvisibilityButtons = new ButtonGroupComponent("images/invisibilityOn.gif",
	    "images/invisibilityEqual.gif",
	    "images/invisibilityOff.gif",
	    false,
	    "Andere sollen nicht sehen, da ich sende bzw. empfange",
	    "Andere drfen sehen, da ich sende bzw. empfange");
	privacyIWantInvisibilityButtons.addActionListener(this);
	gridBagLayout.setConstraints(privacyIWantInvisibilityButtons, buttonGroupConstraints);
	panel.add(privacyIWantInvisibilityButtons);

	panel = new JPanel();
	gridBagLayout.setConstraints(panel, lastPanelConstraints);
	privacyPanel.add(panel);

	titledBorder = new TitledBorder(lineBorder);
	titledBorder.setTitle("Der Partner soll");
	panel.setBorder(titledBorder);
	panel.setLayout(gridBagLayout);

	label = new JLabel("anonym kommunizieren");
	label.setForeground(Color.black);
	gridBagLayout.setConstraints(label, singleLabelConstraints);
	panel.add(label);

	privacyPartnerShouldAnonymityButtons = new ButtonGroupComponent("images/anonymityOn.gif",
	    "images/anonymityEqual.gif",
	    "images/anonymityOff.gif",
	    true,
	    "Der Partner soll anonym sein",
	    "Ich will die Identitt des Partners kennen");
	privacyPartnerShouldAnonymityButtons.addActionListener(this);
	gridBagLayout.setConstraints(privacyPartnerShouldAnonymityButtons, buttonGroupConstraints);
	panel.add(privacyPartnerShouldAnonymityButtons);

	panel = new JPanel();
	titledBorder = new TitledBorder(lineBorder);
	titledBorder.setTitle("Zurcksetzen auf:");
	panel.setBorder(titledBorder);
	panel.setLayout(gridBagLayout);
	gridBagLayout.setConstraints(panel, resetPanelConstraints);
	privacyPanel.add(panel);

	resetHighPrivacyButton = new JButton("hohe Sicherheit");
	resetHighPrivacyButton.addActionListener(this);
	gridBagLayout.setConstraints(resetHighPrivacyButton, leftButtonConstraints);
	panel.add(resetHighPrivacyButton);

	resetMiddlePrivacyButton = new JButton("mittlere Sicherheit");
	resetMiddlePrivacyButton.addActionListener(this);
	gridBagLayout.setConstraints(resetMiddlePrivacyButton, middleButtonConstraints);
	panel.add(resetMiddlePrivacyButton);

	resetLowPrivacyButton = new JButton("niedrige Sicherheit");
	resetLowPrivacyButton.addActionListener(this);
	gridBagLayout.setConstraints(resetLowPrivacyButton, rightButtonConstraints);
	panel.add(resetLowPrivacyButton);

	correctnessPanel = new JPanel();
	correctnessPanel.setLayout(gridBagLayout);

	panel = new JPanel();
	titledBorder = new TitledBorder(lineBorder);
	titledBorder.setTitle("Inhalte");
	panel.setBorder(titledBorder);
	panel.setLayout(gridBagLayout);
	gridBagLayout.setConstraints(panel, panelConstraints);
	correctnessPanel.add(panel);

	label = new JLabel("zustzliche Integrittsmanahmen");
	label.setForeground(Color.black);
	gridBagLayout.setConstraints(label, singleLabelConstraints);
	panel.add(label);

	correctnessContentIntegrityButtons = new ButtonGroupComponent("images/integrityOn.gif",
	    "images/integrityEqual.gif",
	    "images/integrityOff.gif",
	    true,
	    "Ich will zustzliche Integrittsmanahmen",
	    "Die normale Integritt reicht mir");
	correctnessContentIntegrityButtons.addActionListener(this);
	gridBagLayout.setConstraints(correctnessContentIntegrityButtons, buttonGroupConstraints);
	panel.add(correctnessContentIntegrityButtons);

	label = new JLabel(" ");
	label.setForeground(Color.black);
	gridBagLayout.setConstraints(label, firstLabelConstraints);
	panel.add(label);

	label = new JLabel("zustzliche Verfgbarkeitsmanahmen");
	label.setForeground(Color.black);
	gridBagLayout.setConstraints(label, secondLabelConstraints);
	panel.add(label);

	correctnessContentAvailabilityButtons = new ButtonGroupComponent("images/availabilityOn.gif",
	    "images/availabilityEqual.gif",
	    "images/availabilityOff.gif",
	    true,
	    "Ich will zustzliche Verfgbarkeitsmanahmen",
	    "Die normale Verfgbarkeit reicht mir");
	correctnessContentAvailabilityButtons.addActionListener(this);
	gridBagLayout.setConstraints(correctnessContentAvailabilityButtons, buttonGroupConstraints);
	panel.add(correctnessContentAvailabilityButtons);

	panel = new JPanel();
	titledBorder = new TitledBorder(lineBorder);
	titledBorder.setTitle("Ich will");
	panel.setBorder(titledBorder);
	panel.setLayout(gridBagLayout);
	gridBagLayout.setConstraints(panel, panelConstraints);
	correctnessPanel.add(panel);

	label = new JLabel("zurechenbar kommunizieren");
	label.setForeground(Color.black);
	gridBagLayout.setConstraints(label, singleLabelConstraints);
	panel.add(label);

	correctnessIWantAccountabilityButtons = new ButtonGroupComponent("images/accountabilityOn.gif",
	    "images/accountabilityEqual.gif",
	    "images/accountabilityOff.gif",
	    true,
	    "Ich will Beweismittel liefern",
	    "Ich will keine Beweismittel liefern");
	correctnessIWantAccountabilityButtons.addActionListener(this);
	gridBagLayout.setConstraints(correctnessIWantAccountabilityButtons, buttonGroupConstraints);
	panel.add(correctnessIWantAccountabilityButtons);

	label = new JLabel("zustzliche Manahmen, um den Partner");
	label.setForeground(Color.black);
	gridBagLayout.setConstraints(label, firstLabelConstraints);
	panel.add(label);

	label = new JLabel("zu erreichen");
	label.setForeground(Color.black);
	gridBagLayout.setConstraints(label, secondLabelConstraints);
	panel.add(label);

	correctnessIWantReachabilityButtons = new ButtonGroupComponent("images/reachabilityOn.gif",
	    "images/reachabilityEqual.gif",
	    "images/reachabilityOff.gif",
	    true,
	    "Ich will den Partner unbedingt erreichen",
	    "Ich will den Partner erreichen");
	correctnessIWantReachabilityButtons.addActionListener(this);
	gridBagLayout.setConstraints(correctnessIWantReachabilityButtons, buttonGroupConstraints);
	panel.add(correctnessIWantReachabilityButtons);

	panel = new JPanel();
	titledBorder = new TitledBorder(lineBorder);
	titledBorder.setTitle("Der Partner soll");
	panel.setBorder(titledBorder);
	panel.setLayout(gridBagLayout);
	gridBagLayout.setConstraints(panel, lastPanelConstraints);
	correctnessPanel.add(panel);

	label = new JLabel("zurechenbar kommunizieren");
	label.setForeground(Color.black);
	gridBagLayout.setConstraints(label, singleLabelConstraints);
	panel.add(label);

	correctnessPartnerShouldAccountabilityButtons = new ButtonGroupComponent("images/accountabilityOn.gif",
	    "images/accountabilityEqual.gif",
	    "images/accountabilityOff.gif",
	    true,
	    "Der Partner soll Beweismittel liefern",
	    "Der Partner braucht keine Beweismittel zu liefern");
	correctnessPartnerShouldAccountabilityButtons.addActionListener(this);
	gridBagLayout.setConstraints(correctnessPartnerShouldAccountabilityButtons, buttonGroupConstraints);
	panel.add(correctnessPartnerShouldAccountabilityButtons);

	label = new JLabel(" ");
	label.setForeground(Color.black);
	gridBagLayout.setConstraints(label, firstLabelConstraints);
	panel.add(label);

	label = new JLabel("mich erreichen");
	label.setForeground(Color.black);
	gridBagLayout.setConstraints(label, secondLabelConstraints);
	panel.add(label);

	correctnessPartnerShouldReachabilityButtons = new ButtonGroupComponent("images/reachabilityOn.gif",
	    "images/reachabilityEqual.gif",
	    "images/reachabilityOff.gif",
	    false,
	    "Ich will erreichbar sein",
	    "Ich will nicht gestrt werden");
	correctnessPartnerShouldReachabilityButtons.addActionListener(this);
	gridBagLayout.setConstraints(correctnessPartnerShouldReachabilityButtons, buttonGroupConstraints);
	panel.add(correctnessPartnerShouldReachabilityButtons);

	panel = new JPanel();
	titledBorder = new TitledBorder(lineBorder);
	titledBorder.setTitle("Zurcksetzen auf:");
	panel.setBorder(titledBorder);
	panel.setLayout(gridBagLayout);
	gridBagLayout.setConstraints(panel, resetPanelConstraints);
	correctnessPanel.add(panel);

	resetHighCorrectnessButton = new JButton("hohe Sicherheit");
	resetHighCorrectnessButton.addActionListener(this);
	gridBagLayout.setConstraints(resetHighCorrectnessButton, leftButtonConstraints);
	panel.add(resetHighCorrectnessButton);

	resetMiddleCorrectnessButton = new JButton("mittlere Sicherheit");
	resetMiddleCorrectnessButton.addActionListener(this);
	gridBagLayout.setConstraints(resetMiddleCorrectnessButton, middleButtonConstraints);
	panel.add(resetMiddleCorrectnessButton);

	resetLowCorrectnessButton = new JButton("niedrige Sicherheit");
	resetLowCorrectnessButton.addActionListener(this);
	gridBagLayout.setConstraints(resetLowCorrectnessButton, rightButtonConstraints);
	panel.add(resetLowCorrectnessButton);

	tabbedPane = new JTabbedPane();
	tabbedPane.add("Privatheit", privacyPanel);
	tabbedPane.add("Korrektheit", correctnessPanel);
	gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER;
	gridBagConstraints.insets = new Insets(0,0,0,0);
	gridBagConstraints.weightx = 1;
	gridBagConstraints.weighty = 1;
	gridBagConstraints.fill = GridBagConstraints.BOTH;
	gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER;
	gridBagLayout.setConstraints(tabbedPane, gridBagConstraints);
	container.add(tabbedPane);

	okButton = new JButton("OK");
	okButton.addActionListener(this);
	gridBagConstraints.anchor = GridBagConstraints.EAST;
	gridBagConstraints.fill = GridBagConstraints.NONE;
	gridBagConstraints.gridwidth = 1;
	gridBagConstraints.insets = new Insets(7,0,7,0);
	gridBagConstraints.weightx = 0.5;
	gridBagConstraints.weighty = 0;
	gridBagLayout.setConstraints(okButton, gridBagConstraints);
	container.add(okButton);

	cancelButton = new JButton("Abbrechen");
	cancelButton.addActionListener(this);
	gridBagConstraints.anchor = GridBagConstraints.CENTER;
	gridBagConstraints.insets = new Insets(7,7,7,0);
	gridBagConstraints.weightx = 0;
	gridBagLayout.setConstraints(cancelButton, gridBagConstraints);
	container.add(cancelButton);

	helpButton = new JButton("Hilfe");
	helpButton.addActionListener(this);
	gridBagConstraints.anchor = GridBagConstraints.WEST;
	gridBagConstraints.weightx = 0.5;
	gridBagLayout.setConstraints(helpButton, gridBagConstraints);
	container.add(helpButton);

	//prepare button states
	setValues(originalValues);
	privacyPlausibilityCheck();
	correctnessPlausibilityCheck();

	setResizable(false);
	pack();
	Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	Dimension ownDimension = getSize();
	setLocation((screenSize.width - ownDimension.width) / 2,
            (screenSize.height - ownDimension.height) / 2);
	requestFocus();
	show();
    }

    private void setValues(int[] values) {
	privacyContentCipherButtons.setLevel(values[0]);
	privacyContentHideButtons.setLevel(values[1]);
	privacyIWantAnonymityButtons.setLevel(values[2]);
	privacyIWantInvisibilityButtons.setLevel(values[3]);
	privacyPartnerShouldAnonymityButtons.setLevel(values[4]);
	correctnessContentIntegrityButtons.setLevel(values[5]);
	correctnessContentAvailabilityButtons.setLevel(values[6]);
	correctnessIWantAccountabilityButtons.setLevel(values[7]);
	correctnessIWantReachabilityButtons.setLevel(values[8]);
	correctnessPartnerShouldAccountabilityButtons.setLevel(values[9]);
	correctnessPartnerShouldReachabilityButtons.setLevel(values[10]);
    }

    public int[] getValues() {
	int[] values = new int[11];
	values[0] = privacyContentCipherButtons.getLevel();
	values[1] = privacyContentHideButtons.getLevel();
	values[2] = privacyIWantAnonymityButtons.getLevel();
	values[3] = privacyIWantInvisibilityButtons.getLevel();
	values[4] = privacyPartnerShouldAnonymityButtons.getLevel();
	values[5] = correctnessContentIntegrityButtons.getLevel();
	values[6] = correctnessContentAvailabilityButtons.getLevel();
	values[7] = correctnessIWantAccountabilityButtons.getLevel();
	values[8] = correctnessIWantReachabilityButtons.getLevel();
	values[9] = correctnessPartnerShouldAccountabilityButtons.getLevel();
	values[10] = correctnessPartnerShouldReachabilityButtons.getLevel();

	return(values);
    }

    private void setPrivacy(int contentCipherLevel,
			    int contentHideLevel,
			    int iWantAnonymityLevel,
			    int iWantInvisibilityLevel,
			    int partnerShouldAnonymityLevel) {
	privacyContentCipherButtons.setLevel(contentCipherLevel);
	privacyContentHideButtons.setLevel(contentHideLevel);
	privacyIWantAnonymityButtons.setLevel(iWantAnonymityLevel);
	privacyIWantInvisibilityButtons.setLevel(iWantInvisibilityLevel);
	privacyPartnerShouldAnonymityButtons.setLevel(partnerShouldAnonymityLevel);
    }

    private void setPrivacyButtons(boolean highPrivacyButtonState,
				   boolean middlePrivacyButtonState,
				   boolean lowPrivacyButtonState) {
	resetHighPrivacyButton.setEnabled(highPrivacyButtonState);
	resetMiddlePrivacyButton.setEnabled(middlePrivacyButtonState);
	resetLowPrivacyButton.setEnabled(lowPrivacyButtonState);
    }

    private void setCorrectness(int contentIntegrityLevel,
				int contentAvailabilityLevel,
				int iWantAccountabilityLevel,
				int iWantReachabilityLevel,
				int partnerShouldAccountabilityLevel,
				int partnerShouldReachabilityLevel) {
	correctnessContentIntegrityButtons.setLevel(contentIntegrityLevel);
	correctnessContentAvailabilityButtons.setLevel(contentAvailabilityLevel);
	correctnessIWantAccountabilityButtons.setLevel(iWantAccountabilityLevel);
	correctnessIWantReachabilityButtons.setLevel(iWantReachabilityLevel);
	correctnessPartnerShouldAccountabilityButtons.setLevel(partnerShouldAccountabilityLevel);
	correctnessPartnerShouldReachabilityButtons.setLevel(partnerShouldReachabilityLevel);
    }

    private void setCorrectnessButtons(boolean highCorrectnessButtonState,
				       boolean middleCorrectnessButtonState,
				       boolean lowCorrectnessButtonState) {
	resetHighCorrectnessButton.setEnabled(highCorrectnessButtonState);
	resetMiddleCorrectnessButton.setEnabled(middleCorrectnessButtonState);
	resetLowCorrectnessButton.setEnabled(lowCorrectnessButtonState);
    }

    private void privacyPlausibilityCheck() {
	int maxValue = privacyIWantInvisibilityButtons.getLevel();
	privacyContentHideButtons.setMaxLevel(maxValue);
	maxValue = privacyContentHideButtons.getLevel();
	privacyContentCipherButtons.setMaxLevel(maxValue);
    }

    private void privacyPlausibilityCheck(int[] values) {
	int maxValue = values[3];
	if (values[1] > maxValue) values[1] = maxValue;
	maxValue = values[1];
	if (values[0] > maxValue) values[0] = maxValue;
    }

    private void correctnessPlausibilityCheck() {
	// find dependend maximal values for content availability
	int maxValue = correctnessIWantReachabilityButtons.getLevel();
	int nextValue = correctnessPartnerShouldReachabilityButtons.getLevel();
	// higher levels have smaller values (e.g unconditional = 0)
	// that's why we have this odd equation
	if (nextValue < maxValue) maxValue = nextValue;
	correctnessContentAvailabilityButtons.setMaxLevel(maxValue);

	// find dependend maximal values for content integrity
	maxValue = correctnessContentAvailabilityButtons.getLevel();
	nextValue = correctnessIWantAccountabilityButtons.getLevel();
	if (nextValue < maxValue) maxValue = nextValue;
	nextValue = correctnessPartnerShouldAccountabilityButtons.getLevel();
	if (nextValue < maxValue) maxValue = nextValue;
	correctnessContentIntegrityButtons.setMaxLevel(maxValue);
    }

    private void correctnessPlausibilityCheck(int[] values) {
	// find dependend maximal values for content availability
	int maxValue = values[8];
	int nextValue = values[10];
	// higher levels have smaller values (e.g unconditional = 0)
	// that's why we have this odd equation
	if (nextValue < maxValue) maxValue = nextValue;
	values[6] = maxValue;

	// find dependend maximal values for content integrity
	maxValue = values[6];
	nextValue = values[7];
	if (nextValue < maxValue) maxValue = nextValue;
	nextValue = values[9];
	if (nextValue < maxValue) maxValue = nextValue;
	values[5] = maxValue;
    }

    private boolean showAnonymityMonotonyCheckDialog(ButtonGroupComponent buttonsToCheck,
						     int originalValue,
						     String message,
						     ImageIcon icon) {
	Object[] options = {"Anpassung vornehmen","Zurcksetzen","Abbrechen"};
	int returnValue = JOptionPane.showOptionDialog(this,
	    message,
	    "Hinweis zur Anonymittseinstellung",
	    JOptionPane.YES_NO_CANCEL_OPTION,
	    JOptionPane.WARNING_MESSAGE,
	    icon,
	    options,
	    options[0]);
	if(returnValue == JOptionPane.YES_OPTION) {
	    // ok, lets do it
	    return true;
	} else if(returnValue == JOptionPane.NO_OPTION) {
	    // set values back to originalValue
	    buttonsToCheck.setLevel(originalValue);
	}
	return false;
    }

    private int showCorrectnessMonotonyCheckDialog(String message, ImageIcon icon) {
	Object[] options = {"Transaktion","Aktion","Abbrechen"};
	int returnValue = JOptionPane.showOptionDialog(this,
	    message,
	    "Hinweis zur Zurechenbarkeitseinstellung",
	    JOptionPane.YES_NO_CANCEL_OPTION,
	    JOptionPane.WARNING_MESSAGE,
	    icon,
	    options,
	    options[0]);
	if(returnValue == JOptionPane.YES_OPTION) {
	    // change values for whole transaction
	    return CHANGE_TRANSACTION;
	} else if(returnValue == JOptionPane.NO_OPTION) {
	    // change only this action
	    return CHANGE_ACTION;
	}
	return ABORT;
    }

    private boolean monotonyCheck() {
	// checkings for privacy
	DefaultMutableTreeNode previousNode = selectedNode.getPreviousSibling();
	if (previousNode != null) {
	    int[] previousValues = ((SecuritySettings)previousNode.getUserObject()).getUserDefinedValues();
	    // check previous values
	    if (privacyIWantAnonymityButtons.getLevel() < previousValues[2]) {
		boolean makeChanges = showAnonymityMonotonyCheckDialog(privacyIWantAnonymityButtons,
		    originalValues[2],
		    "Sie stellen Ihre Anonymitt hher als in vorherigen\n" +
		    "Aktionen der Transaktion ein. Fr diese Aktionen wird Ihre\n" +
		    "Anonymitt mindestens auf den hier eingestellten Wert gesetzt.",
		    new ImageIcon("images/anonymityOn.gif"));
		if(!makeChanges) return false;
		makePreviousOwnPrivacyChanges = true;
	    }
	    if (privacyPartnerShouldAnonymityButtons.getLevel() < previousValues[4]) {
		boolean makeChanges = showAnonymityMonotonyCheckDialog(privacyPartnerShouldAnonymityButtons,
		    originalValues[4],
		    "Sie stellen die Anonymitt Ihres Partners hher als in vorherigen\n" +
		    "Aktionen der Transaktion ein. Fr diese Aktionen wird die Anonymitt\n" +
		    "Ihres Partners mindestens auf den hier eingestellten Wert gesetzt.",
		    new ImageIcon("images/anonymityOn.gif"));
		if(!makeChanges) return false;
		makePreviousPartnerPrivacyChanges = true;
	    }
	}

	DefaultMutableTreeNode nextNode = selectedNode.getNextSibling();
	if (nextNode != null) {
	    int[] nextValues = ((SecuritySettings)nextNode.getUserObject()).getUserDefinedValues();
	    // check next values
	    if (privacyIWantAnonymityButtons.getLevel() > nextValues[2]) {
		boolean makeChanges = showAnonymityMonotonyCheckDialog(privacyIWantAnonymityButtons,
		    originalValues[2],
		    "Sie stellen Ihre Anonymitt niedriger als in nachfolgenden\n" +
		    "Aktionen der Transaktion ein. Fr diese Aktionen wird Ihre\n" +
		    "Anonymitt hchstens auf den hier eingestellten Wert gesetzt.",
		    new ImageIcon("images/anonymityOff.gif"));
		if(!makeChanges) return false;
		makeNextOwnPrivacyChanges = true;
	    }
	    if (privacyPartnerShouldAnonymityButtons.getLevel() > nextValues[4]) {
		boolean makeChanges = showAnonymityMonotonyCheckDialog(privacyPartnerShouldAnonymityButtons,
		    originalValues[4],
		    "Sie stellen die Anonymitt Ihres Partners niedriger als in nachfolgenden\n" +
		    "Aktionen der Transaktion ein. Fr diese Aktionen wird die Anonymitt\n" +
		    "Ihres Partners hchstens auf den hier eingestellten Wert gesetzt.",
		    new ImageIcon("images/anonymityOff.gif"));
		if(!makeChanges) return false;
		makeNextPartnerPrivacyChanges = true;
	    }
	}

	// checkings for accountability
	if (correctnessIWantAccountabilityButtons.getLevel() < originalValues[7]) {
	    int returnValue = showCorrectnessMonotonyCheckDialog(
		"Sie stellen Ihre Zurechenbarkeit hher ein. Damit liefern\n" +
		"Sie Beweismittel. Soll der eingestellte Wert der gesamten\n" +
		"Transaktion oder nur dieser Aktion zugewiesen werden?",
		new ImageIcon("images/accountabilityOn.gif"));
	    if (returnValue == CHANGE_TRANSACTION) changeOwnTransaction = true;
	    if (returnValue == ABORT) return false;
	}
	if (correctnessIWantAccountabilityButtons.getLevel() > originalValues[7]) {
	    int returnValue = showCorrectnessMonotonyCheckDialog(
		"Sie stellen Ihre Zurechenbarkeit niedriger ein. Damit\n" +
		"liefern Sie keine Beweismittel. Soll der eingestellte Wert der\n" +
		"gesamten Transaktion oder nur dieser Aktion zugewiesen werden?",
		new ImageIcon("images/accountabilityOff.gif"));
	    if (returnValue == CHANGE_TRANSACTION) changeOwnTransaction = true;
	    if (returnValue == ABORT) return false;
	}
	if (correctnessPartnerShouldAccountabilityButtons.getLevel() < originalValues[9]) {
	    int returnValue = showCorrectnessMonotonyCheckDialog(
		"Sie stellen die Zurechenbarkeit Ihres Partners hher ein.\n" +
		"Damit liefert er Beweismittel. Soll der eingestellte Wert der\n" +
		"gesamten Transaktion oder nur dieser Aktion zugewiesen werden?",
		new ImageIcon("images/accountabilityOn.gif"));
	    if (returnValue == CHANGE_TRANSACTION) changePartnerTransaction = true;
	    if (returnValue == ABORT) return false;
	}
	if (correctnessPartnerShouldAccountabilityButtons.getLevel() > originalValues[9]) {
	    int returnValue = showCorrectnessMonotonyCheckDialog(
		"Sie stellen die Zurechenbarkeit Ihres Partners niedriger ein.\n" +
		"Damit liefert er keine Beweismittel. Soll der eingestellte Wert\n" +
		"der gesamten Transaktion oder nur dieser Aktion zugewiesen werden?",
		new ImageIcon("images/accountabilityOff.gif"));
	    if (returnValue == CHANGE_TRANSACTION) changePartnerTransaction = true;
	    if (returnValue == ABORT) return false;
	}
	return true;
    }

    private void changeMonotonySettings() {
	// privacy changings
	int[] newValues = getValues();
	((SecuritySettings)selectedNode.getUserObject()).setUserDefinedValues(newValues);
	if (makePreviousOwnPrivacyChanges || makePreviousPartnerPrivacyChanges) {
	    boolean done = false;
	    DefaultMutableTreeNode previousNode = selectedNode.getPreviousSibling();
	    if (previousNode == null) done = true;
	    while (!done) {
		SecuritySettings previousNodeSecuritySettings = (SecuritySettings)previousNode.getUserObject();
		int[] previousNodeValues = previousNodeSecuritySettings.getUserDefinedValues();
		if (makePreviousOwnPrivacyChanges) {
		    if (newValues[2] < previousNodeValues[2]) {
			previousNodeValues[2] = newValues[2];
			previousNodeSecuritySettings.setUserDefinedValues(previousNodeValues);
		    } else {
			done = true;
		    }
		}
		if (makePreviousPartnerPrivacyChanges) {
		    if (newValues[4] < previousNodeValues[4]) {
			previousNodeValues[4] = newValues[4];
			previousNodeSecuritySettings.setUserDefinedValues(previousNodeValues);
		    } else {
			done = true;
		    }
		}
		previousNode = previousNode.getPreviousSibling();
		if (previousNode == null) done = true;
	    }
	}
	if (makeNextOwnPrivacyChanges || makeNextPartnerPrivacyChanges) {
	    boolean done = false;
	    DefaultMutableTreeNode nextNode = selectedNode.getNextSibling();
	    if (nextNode == null) done = true;
	    while (!done) {
		SecuritySettings nextNodeSecuritySettings = (SecuritySettings)nextNode.getUserObject();
		int[] nextNodeValues = nextNodeSecuritySettings.getUserDefinedValues();
		if (makeNextOwnPrivacyChanges) {
		    if (newValues[2] > nextNodeValues[2]) {
			nextNodeValues[2] = newValues[2];
			nextNodeSecuritySettings.setUserDefinedValues(nextNodeValues);
		    } else {
			done = true;
		    }
		}
		if (makeNextPartnerPrivacyChanges) {
		    if (newValues[4] > nextNodeValues[4]) {
			nextNodeValues[4] = newValues[4];
			nextNodeSecuritySettings.setUserDefinedValues(nextNodeValues);
		    } else {
			done = true;
		    }
		}
		nextNode = nextNode.getNextSibling();
		if (nextNode == null) done = true;
	    }
	}

	// correctness changings
	if (changeOwnTransaction || changePartnerTransaction) {
	    Enumeration actions = selectedNode.getParent().children();
	    while(actions.hasMoreElements()) {
		DefaultMutableTreeNode node = (DefaultMutableTreeNode)actions.nextElement();
		SecuritySettings nodeSecuritySettings = (SecuritySettings)node.getUserObject();
		int[] nodeValues = nodeSecuritySettings.getUserDefinedValues();
		if (changeOwnTransaction) {
		    nodeValues[7] = newValues[7];
		}
		if (changePartnerTransaction) {
		    nodeValues[9] = newValues[9];
		}
		privacyPlausibilityCheck(nodeValues);
		correctnessPlausibilityCheck(nodeValues);
		nodeSecuritySettings.setUserDefinedValues(nodeValues);
	    }
	}
    }

    public void actionPerformed(ActionEvent event) {
	if (event.getSource() == okButton) {
	    if (selectedNode.isLeaf()) {
		// monotony check is only for actions (tree leafes)
		makePreviousOwnPrivacyChanges = false;
		makePreviousPartnerPrivacyChanges = false;
		makeNextOwnPrivacyChanges = false;
		makeNextPartnerPrivacyChanges = false;
		changeOwnTransaction = false;
		changePartnerTransaction = false;
		if (monotonyCheck() == true) {
		    changeMonotonySettings();
		    dispose();
		}
	    } else {
		// don't make monotony check, just set values
		((SecuritySettings)selectedNode.getUserObject()).setUserDefinedValues(getValues());
		// this dialog does not block any more, because it's no longer modal
		// that's why the parent dialog can't wait for the return from show() any more
		// so we have to trigger the subTreeSecurityChangings from here!
		communicationSecurityDialog.setSubTreeSecurity(SecuritySettings.USER_DEFINED);
		dispose();
	    }
	} else if (event.getSource() == cancelButton) {
	    dispose();
	} else if (event.getSource() == helpButton) {
	    help.vanishWindow();
	    help = new Help("file:///"+System.getProperty("ssonet_help")+"goals/HelpSet.hs");
	    help.viewPage("expert");
	} else if (event.getSource() == resetHighPrivacyButton) {
	    setPrivacy(1,1,1,1,5);
	    setPrivacyButtons(false, true, true);
	    privacyPlausibilityCheck();
	} else if (event.getSource() == resetMiddlePrivacyButton) {
	    setPrivacy(2,2,2,2,4);
	    setPrivacyButtons(true, false, true);
	    privacyPlausibilityCheck();
	} else if (event.getSource() == resetLowPrivacyButton) {
	    setPrivacy(4,4,4,4,2);
	    setPrivacyButtons(true, true, false);
	    privacyPlausibilityCheck();
	} else if (event.getSource() == resetHighCorrectnessButton) {
	    setCorrectness(1,1,5,1,1,3);
	    setCorrectnessButtons(false, true, true);
	    correctnessPlausibilityCheck();
	} else if (event.getSource() == resetMiddleCorrectnessButton) {
	    setCorrectness(2,2,4,2,2,3);
	    setCorrectnessButtons(true, false, true);
	    correctnessPlausibilityCheck();
	} else if (event.getSource() == resetLowCorrectnessButton) {
	    setCorrectness(4,4,2,3,4,3);
	    setCorrectnessButtons(true, true, false);
	    correctnessPlausibilityCheck();
	} else if (event.getSource() == privacyIWantInvisibilityButtons ||
	    event.getSource() == privacyContentHideButtons) {
	    setPrivacyButtons(true, true, true);
	    privacyPlausibilityCheck();
	} else if (event.getSource() == privacyIWantAnonymityButtons ||
	    event.getSource() == privacyPartnerShouldAnonymityButtons) {
	    setPrivacyButtons(true, true, true);
	} else if (event.getSource() == privacyContentCipherButtons) {
	    setPrivacyButtons(true, true, true);
	} else if (event.getSource() == correctnessContentIntegrityButtons) {
	    setCorrectnessButtons(true, true, true);
	} else if (event.getSource() == correctnessContentAvailabilityButtons ||
	    event.getSource() == correctnessIWantAccountabilityButtons ||
	    event.getSource() == correctnessIWantReachabilityButtons ||
	    event.getSource() == correctnessPartnerShouldAccountabilityButtons ||
	    event.getSource() == correctnessPartnerShouldReachabilityButtons) {
	    setCorrectnessButtons(true, true, true);
	    correctnessPlausibilityCheck();
	}
    }
}
