/*
 * smi/baseConf/BaseConfiguration.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package smi.baseConf;

import cryptix.provider.rsa.RawRSAPublicKey;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.Dimension;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.File;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.SecureRandom;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import smi.awt.AppletFrame;
import smi.awt.JPreferenceList;
import smi.awt.Ping;
import smi.baseConf.RatedString;
import smi.baseConf.CancelException;
import smi.mechanisms.Mechanism;
import smi.mechanisms.MechanismConfiguration;
import smi.mechanisms.ParameterNotCompatibleException;
import smi.io.IOStream;

import ssonet.SSONETConstants;
import ssonet.KeyExchange;
import ssonet.IdentityCheck;
import ssonet.Preload;
import help.Help;
import gateway.GatewayWindow;
import smi.user.*;

import smi.baseConf.ConfigurationAnonymity;
import smi.baseConf.ConfigurationConfidentiality;
import smi.baseConf.ConfigurationAccountability;
import smi.baseConf.ConfigurationIntegrity;

import sun.security.util.BigInt;
import sun.security.util.DerValue;
import sun.security.util.DerOutputStream;
import sun.security.x509.AlgorithmId;
import sun.security.x509.X500Name;
import sun.security.x509.X500Signer;
import sun.security.x509.X509Cert;
import sun.security.x509.X509Key;

final public class BaseConfiguration extends JApplet implements ActionListener {

    private JButton cancelButton;
    private JTabbedPane tabbedPane, accoutabilityTabbedPane;
    private String[] tabHeadKey = {
	"CONFIDENTIALITY",
	"ANONYMITY",
	"INTEGRITY",
	"ACCOUNTABILITY",
    };


    private JMenu menu[] = new JMenu[4];
    private String menuTextKey[] = {
	"SETTINGS",
	"USER",
	"HELP",
	"LANGUAGE"
    };

    private JMenuItem menuitem[] = new JMenuItem[7];
    private String menuitemTextKey[] =
    {
	"PERFORMANCE_TEST",
	"RATING",
	"SELECT/CREATE USER",
	"USER PROPERTIES",
	"HELP",
	"INFO",
	"PROVIDER"
    };

    private JCheckBoxMenuItem checkBoxMenuItem[] = new JCheckBoxMenuItem[2];
    private String checkBoxMenuItemTextKey[] = {"GERMAN","ENGLISH"};

    private static String PERFORMANCE_TEST = "PERFORMANCE_TEST";
    private static String INFO = "INFO";
    private static String HELP = "HELP";
    private static String RATING = "RATING";
    private static String PROVIDER = "PROVIDER";
    private static String CREATE_SELECT_USER = "CREATE/SELECT USER";
    private static String USER_PROPERTIES = "USER PROPERTIES";
    private static String HELPANONYMITY = "help/hilfetexte/hilfeAnonymitt.html";
    private static String HELPCONFIDENTIALITY = "help/hilfetexte/hilfeVertraulichkeit.html";
    private static String HELPINTEGRITY = "help/hilfetexte/hilfeVertraulichkeit.html";
    private static String HELPACCOUNTABILITY = "help/hilfetexte/hilfeZurechenbarkeit.html";

    private JLabel confidetialityPanelLabel, integrityPanelLabel, anonymityPanelLabel;

    private JCheckBox askBeforeSignCheckBox;

    private static Help hp = null;
    private static ResourceBundle resource = null;


    private String changeConfigurationButtonTextKey[] =
    {
	"DETAIL",
	"DETAIL",
	"DETAIL",
	"DETAIL"
    };
    private JPanel userPanel;
    private JLabel userNameLabel, userNameShowLabel, userDirLabel, userDirShowLabel;

    private AppletFrame appletFrame = null;
    private Hashtable baseConfiguration;
    private Hashtable userProperties;
// The four Panels
    private ConfigurationAnonymity caPanel;
    private ConfigurationConfidentiality ccPanel;
    private ConfigurationAccountability cacPanel;
    private ConfigurationIntegrity ciPanel;

    private static Hashtable config;
    private static int which;

    private class ButtonPanel extends JPanel implements ActionListener {
	private JButton okButton;
	public ButtonPanel() {
	    super();
	    GridBagConstraints gridBagConstraints = new GridBagConstraints();
	    GridBagLayout gridBagLayout = new GridBagLayout();
	    this.setLayout(gridBagLayout);

	    gridBagConstraints.insets = new Insets(10,10,10,10);
	    gridBagConstraints.anchor = GridBagConstraints.CENTER;
	    gridBagConstraints.gridx = 0;
	    gridBagConstraints.gridy = 0;
	    gridBagConstraints.gridwidth = 1;
	    gridBagConstraints.gridheight = 1;
	    gridBagConstraints.fill = GridBagConstraints.NONE;
	    gridBagConstraints.ipadx = 0;
	    gridBagConstraints.ipady = 0;
	    gridBagConstraints.weightx = 0.333;
	    gridBagConstraints.weighty = 1;

	    this.okButton = new JButton("     Ok     ");
	    gridBagLayout.setConstraints(okButton, gridBagConstraints);
	    this.add(okButton);
	    okButton.addActionListener(this);

	    gridBagConstraints.gridx = 1;
	    cancelButton = new JButton();
	    gridBagLayout.setConstraints(cancelButton, gridBagConstraints);
	    this.add(cancelButton);
	    cancelButton.addActionListener(this);
	}

	public void actionPerformed(ActionEvent event) {
	    if(event.getSource() == okButton) {
                caPanel.writeConfig();
                ccPanel.writeConfig();
                cacPanel.writeConfig();
                ciPanel.writeConfig();
	    }
	    appletFrame.dispose();
	    System.exit(0);
	}
    }

    private void changeResource(ResourceBundle resource) {

	cancelButton.setText(resource.getString("CANCEL"));
	for(int i = 0; i < tabHeadKey.length; i++)
	    tabbedPane.setTitleAt(i, resource.getString(tabHeadKey[i]));

	appletFrame.setTitle(resource.getString("BASE CONFIGURATION"));

	for(int i = 0; i < menu.length; i++)
	    menu[i].setText(resource.getString(menuTextKey[i]));
	for(int i = 0; i < menuitem.length; i++)
	    menuitem[i].setText(resource.getString(menuitemTextKey[i]));
	for(int i = 0; i < checkBoxMenuItem.length; i++)
	    checkBoxMenuItem[i].setText(resource.getString(checkBoxMenuItemTextKey[i]));

	//user panel
	((TitledBorder)userPanel.getBorder()).setTitle(resource.getString("USER"));
	userNameLabel.setText(resource.getString("NAME")+":");
	userDirLabel.setText(resource.getString("USER DIRECTORY")+":");

	validate();
    }




// Read the actual configuration out of the Panels
    public static void help(int which)
    {
    try{
    switch(which){
         case 1:
         config = ConfigurationConfidentiality.getConfigurationConfidentiality();
         break;
         case 2:
         config = ConfigurationIntegrity.getConfigurationIntegrity();
         break;
         case 3:
         config = ConfigurationAccountability.getConfigurationAccountability();
         break;
                 }//switch

    }catch(Exception e)
    {
    e.printStackTrace();
    }
    }

    /** get a preference list . . . .
     */
    public static RatedString[] getPreferenceList(int securityGoal)
    {
	try {

	    String[] preferenceList = null;
	    if(securityGoal == SSONETConstants.CONFIDENTIALITY){
              help(1);
              preferenceList = (String[]) config.get("preflist.confidentiality") ;
              }
	    else if(securityGoal == SSONETConstants.INTEGRITY){
               help(2);
               preferenceList = (String[]) config.get("preflist.integrity");
               }
	    else if(securityGoal == SSONETConstants.ACCOUNTABILITY_SIGN){
                help(3);
		preferenceList = (String[]) config.get("preflist.accountability.sign");
                }
	    else{
		return null;
                }
	    return RatedString.toRatedString(preferenceList);
	} catch(Exception e) {
	    e.printStackTrace();
	    return null;
	}
    }

    private void addMenu() {
	JMenuBar menuBar = new JMenuBar();

	for (int i = 0; i < menu.length; i++)
	    menu[i] = new JMenu();
	for (int i = 0; i < menuitem.length; i++)
	    menuitem[i] = new JMenuItem();
	ButtonGroup languageButtonGroup = new ButtonGroup();
	for (int i = 0; i < checkBoxMenuItem.length; i++) {
	    checkBoxMenuItem[i] = new JCheckBoxMenuItem();
	    languageButtonGroup.add(checkBoxMenuItem[i]);
	}

	checkBoxMenuItem[0].setState(true);

	menu[0].add(menu[3]);                   // Settings ... Language
	menu[3].add(checkBoxMenuItem[0]);           // Language... German
	checkBoxMenuItem[0].addActionListener(this);
	menu[3].add(checkBoxMenuItem[1]);           // Language... English
	checkBoxMenuItem[1].addActionListener(this);

	menu[0].add(menuitem[6]);               //Settings ... Provider
	menuitem[6].addActionListener(this);
	menuitem[6].setActionCommand(PROVIDER);

	menu[0].add(menuitem[0]);               //Settings ... Performancetest
	menuitem[0].addActionListener(this);
	menuitem[0].setActionCommand(PERFORMANCE_TEST);

	menu[0].add(menuitem[1]);               //Settings ... Rating
	menuitem[1].addActionListener(this);
	menuitem[1].setActionCommand(RATING);
	menuBar.add(menu[0]);

	menu[1].add(menuitem[2]);               //User --> Create/Select user
	menuitem[2].addActionListener(this);
	menuitem[2].setActionCommand(CREATE_SELECT_USER);

	menu[1].add(menuitem[3]);               //User --> User Properties
	menuitem[3].addActionListener(this);
	menuitem[3].setActionCommand(USER_PROPERTIES);
	menuBar.add(menu[1]);

	menu[2].add(menuitem[4]);               //Help ... Help
	menuitem[4].addActionListener(this);
	menuitem[4].setActionCommand(HELP);
	menuBar.add(menu[2]);

	menu[2].add(menuitem[5]);               //Help ... Info
	menuitem[5].addActionListener(this);
	menuitem[5].setActionCommand(INFO);
	menuBar.add(menu[2]);

	setJMenuBar(menuBar);
    }



    private String getMechanismsDir() {
	return "./smi/mechanisms";
    }

    public void init() {
	// preload of the Cryptix Securityprovider
	Preload preload = new Preload();
	try{
	    preload.join();
	}catch(Exception e){}
	// select user, if property is not set
	if(System.getProperty("ssonet_userdir") == null){
	    UserSelectDialog usd = new UserSelectDialog(new JDialog(), true, Locale.GERMAN);
	    usd.show();
	    if(usd.getSelectedUser() == null){
		System.exit(0);
	    } else {
		System.setProperty("ssonet_userdir",usd.getSelectedUser());
	    }
	}
	// initialize frame
	appletFrame = new AppletFrame("BASE CONFIGURATION");
	initialize();
	start();
	appletFrame.getContentPane().add(this);
	appletFrame.pack();
	appletFrame.setSize(appletFrame.getSize().width+70, appletFrame.getSize().height + 50);
	appletFrame.center();
	appletFrame.show();
    }

    private void initialize() {
	setBackground(Color.lightGray);

	addMenu();

	GridBagLayout gridBagLayout = new GridBagLayout();
	GridBagConstraints gridBagConstraints = new GridBagConstraints();

	tabbedPane = new JTabbedPane(JTabbedPane.TOP);

	// info about the user
	userPanel = new JPanel(new GridLayout(2,2));
	userPanel.setBorder(new TitledBorder(LineBorder.createBlackLineBorder(),"dummy"));
	userNameLabel = new JLabel("");
	userPanel.add(userNameLabel);
	userNameShowLabel = new JLabel("");
	userPanel.add(userNameShowLabel);
	userDirLabel = new JLabel("");
	userPanel.add(userDirLabel);
	userDirShowLabel = new JLabel("");
	userPanel.add(userDirShowLabel);

        ccPanel = new ConfigurationConfidentiality(appletFrame, System.getProperty("ssonet_cfg")+ "users/"
        ,System.getProperty("ssonet_userdir"),new Hashtable());
        tabbedPane.add(ccPanel);
        caPanel = new ConfigurationAnonymity(appletFrame, System.getProperty("ssonet_cfg")+ "users/"
        ,System.getProperty("ssonet_userdir"), new Hashtable());
        tabbedPane.add(caPanel,getClass().getResource("images/red.gif"));
        ciPanel = new ConfigurationIntegrity(appletFrame, System.getProperty("ssonet_cfg")+ "users/"
         ,System.getProperty("ssonet_userdir"), new Hashtable());
        tabbedPane.add(ciPanel,getClass().getResource("images/green.gif"));
        cacPanel = new ConfigurationAccountability(appletFrame, System.getProperty("ssonet_cfg")+ "users/"
        ,System.getProperty("ssonet_userdir"), new Hashtable());
        tabbedPane.add(cacPanel, getClass().getResource("images/red.gif"));
	tabbedPane.setIconAt(0, new ImageIcon(getClass().getResource("images/red.gif")));
	tabbedPane.setIconAt(1, new ImageIcon(getClass().getResource("images/red.gif")));
	tabbedPane.setIconAt(2, new ImageIcon(getClass().getResource("images/green.gif")));
	tabbedPane.setIconAt(3, new ImageIcon(getClass().getResource("images/green.gif")));


	getContentPane().setLayout(new BorderLayout());
	getContentPane().add("Center", tabbedPane);
	getContentPane().add("South", new ButtonPanel());
	getContentPane().add("North", userPanel);
	getContentPane().add("West", new JPanel());
	getContentPane().add("East", new JPanel());

	resource = ResourceBundle.getBundle("smi.baseConf.locale.Resources", Locale.GERMAN);
	changeResource(resource);
        String help = System.getProperty("ssonet_userdir");
        int hilf = help.length();
	changeUser(help.substring(0,hilf));
    }

    public void actionPerformed(ActionEvent event) {
	Object object = event.getSource();

        if(object == checkBoxMenuItem[0]) {
	    // language: german
	    resource = ResourceBundle.getBundle("smi.baseConf.locale.Resources", Locale.GERMAN);
	    changeResource(resource);
            caPanel.changeLanguage(Locale.GERMAN);
            ccPanel.changeLanguage(Locale.GERMAN);
            ciPanel.changeLanguage(Locale.GERMAN);
            cacPanel.changeLanguage(Locale.GERMAN);
	}
	else if(object == checkBoxMenuItem[1]) {
	    // language: english
	    resource = ResourceBundle.getBundle("smi.baseConf.locale.Resources", Locale.ENGLISH);
	    changeResource(resource);
            caPanel.changeLanguage(Locale.ENGLISH);
            ccPanel.changeLanguage(Locale.ENGLISH);
            ciPanel.changeLanguage(Locale.ENGLISH);
            cacPanel.changeLanguage(Locale.ENGLISH);
	} else {
	    if(event.getActionCommand().equals(HELP)) {
		viewHelp("GRUNDKONFIGURATION","");
	    }
	    if(event.getActionCommand().equals(INFO))
		new AboutBaseConfiguration(appletFrame);
	    else if(event.getActionCommand().equals(PERFORMANCE_TEST))
		(new PerformanceTest()).start();
	    else if(event.getActionCommand().equals(RATING))
		new rating.Rating();
	    else if(event.getActionCommand().equals(PROVIDER)) {
		GatewayWindow aGateway = new GatewayWindow();
		aGateway.showGatewayFrame();
	    } else if(event.getActionCommand().equals(CREATE_SELECT_USER)) {
		// show user selection dialog and load data from the new
		UserSelectDialog usd = new UserSelectDialog(new JDialog(), true, resource.getLocale());
		usd.show();
		if(usd.getSelectedUser() != null){
		    int choise = JOptionPane.showConfirmDialog(this,resource.getString("SAVE SETTINGS FOR USER")
			+(String)userProperties.get(UserPropertiesDialog.USER_PROPERTIES_USERID)+"?" ,
			resource.getString("QUESTION"),
			JOptionPane.YES_NO_CANCEL_OPTION,JOptionPane.QUESTION_MESSAGE);
		    switch(choise){
		    case JOptionPane.YES_OPTION:
			// Save settings first

                        ccPanel.confidentialityConfiguration.put("preflist.confidentality",
                        ccPanel.confidentialityPreferenceList.getPreferenceListItems());
                        ccPanel.writeConfig();
                        cacPanel.accountabilityConfiguration.put("preflist.accountability.sign",
                        cacPanel.accountabilitySignPreferenceList.getPreferenceListItems());
                        cacPanel.accountabilityConfiguration.put("preflist.accountability.accept",
                        cacPanel.accountabilityAcceptSignPreferenceList.getPreferenceListItems());
                        cacPanel.writeConfig();

                        caPanel.anonymityConfiguration.put("preflist.anonymity",
                        caPanel.anonymityPreferenceList.getPreferenceListItems());
                        caPanel.writeConfig();

                        ciPanel.integrityConfiguration.put("preflist.integrity",
                        ciPanel.integrityPreferenceList.getPreferenceListItems());
                        ciPanel.writeConfig();

			changeUser(usd.getSelectedUser());
			break;
		    case JOptionPane.NO_OPTION:
			changeUser(usd.getSelectedUser());
			break;
		    // CANCEL: do nothing
		    }
		}
	    } else if(event.getActionCommand().equals(USER_PROPERTIES)) {
		// show dialog to change user properties
		UserPropertiesDialog upd = new UserPropertiesDialog(new JDialog(), true, resource.getLocale(),
		    System.getProperty("ssonet_userdir"), false);
		upd.show();
	    }
	}
    }

    /**
     * set a new user and load its properties and its baseconfiguration
     */
    private void changeUser(String userDirName){
	System.setProperty("ssonet_userdir",userDirName);
	// read user properties from disk
	try{
	    FileInputStream fis = new FileInputStream(System.getProperty("ssonet_cfg")+"users/"
		    +userDirName + "/userproperties.dat");
            System.out.println(System.getProperty("ssonet_cfg")+"users/x"
		    +userDirName + "/xuserproperties.dat");
	    ObjectInputStream ois = new ObjectInputStream(fis);
	    userProperties = (Hashtable)ois.readObject();
	    ois.close();
	    fis.close();
	}
	catch (Exception e){
	    System.out.println("No userproperties for userdirectory "+userDirName+" found.");
	    return;
	}
	int i = ((Integer)userProperties.get(UserPropertiesDialog.USER_PROPERTIES_LANGUAGE)).intValue();
	switch (i){
	case 0: resource = ResourceBundle.getBundle("smi.baseConf.locale.Resources",Locale.GERMAN);
	    checkBoxMenuItem[0].setState(true);
            caPanel.changeLanguage(Locale.GERMAN);
            ciPanel.changeLanguage(Locale.GERMAN);
            ccPanel.changeLanguage(Locale.GERMAN);
            cacPanel.changeLanguage(Locale.GERMAN);
	    break;
	case 1: resource = ResourceBundle.getBundle("smi.baseConf.locale.Resources",Locale.ENGLISH);
	    checkBoxMenuItem[1].setState(true);
            caPanel.changeLanguage(Locale.ENGLISH);
            ciPanel.changeLanguage(Locale.ENGLISH);
            ccPanel.changeLanguage(Locale.ENGLISH);
            cacPanel.changeLanguage(Locale.ENGLISH);
	}
	changeResource(resource);
	userNameShowLabel.setText((String)userProperties.get(UserPropertiesDialog.USER_PROPERTIES_USERID));
	userDirShowLabel.setText(new File(System.getProperty("ssonet_cfg")+"users/"+userDirName).getAbsolutePath());
	// load baseconfig
      ccPanel.changeUser(System.getProperty("ssonet_userdir")+"/");
      ccPanel.readConfig();
      ccPanel.confidentialityPreferenceList
      .setPreferenceListData((String[])ccPanel.confidentialityConfiguration.get("preflist.confidentiality"));
      ciPanel.changeUser(System.getProperty("ssonet_userdir")+"/");
      ciPanel.readConfig();
      ciPanel.integrityPreferenceList.setPreferenceListData((String[])ciPanel.integrityConfiguration.get(
      "preflist.integrity"));
      caPanel.changeUser(System.getProperty("ssonet_userdir")+"/");
      caPanel.readConfig();
      System.out.println("changeUser");
      caPanel.anonymityPreferenceList.setPreferenceListData((String[])caPanel.anonymityConfiguration.get(
      "preflist.anonymity"));
      cacPanel.changeUser(System.getProperty("ssonet_userdir")+"/");
      cacPanel.readConfig();
      cacPanel.accountabilitySignPreferenceList.setPreferenceListData(
      (String[])cacPanel.accountabilityConfiguration.get("preflist.accountability.sign"));
      cacPanel.accountabilityAcceptSignPreferenceList.setPreferenceListData((String[])
      cacPanel.accountabilityConfiguration.get("preflist.accountability.accept"));
    }

    public static void viewHelp(String str1, String str2) {
	if (hp != null) {
	    hp.vanishWindow();
	}
	hp = new Help(System.getProperty("ssonet_help")+"gk/HelpSet.hs");
	hp.viewPage (str1, str2);
    }

    public static ResourceBundle getLangResources() {
	if (resource == null)  {
	    resource = ResourceBundle.getBundle("smi.baseConf.locale.Resources", Locale.GERMAN);
	}
	return resource;
    }

    /**
     * start the Base-Configuration
     * the first commandline argument is the <userdir>. If it is not given, a user selection dialog
     * is shown.
     */
    public static void main(String  argv[]) {
	if(argv.length > 0){
	    System.setProperty("ssonet_userdir",argv[0]);

	}
	BaseConfiguration baseConfiguration = new BaseConfiguration();
	baseConfiguration.init();
    }
}
