/*
 * smi/baseConf/PerformanceTest.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package smi.baseConf;

import cryptix.provider.key.BlowfishKeyGenerator;
import cryptix.provider.key.CAST5KeyGenerator;
import cryptix.provider.key.DESKeyGenerator;
import cryptix.provider.key.DES_EDE3KeyGenerator;
import cryptix.provider.key.IDEAKeyGenerator;
import cryptix.provider.key.LOKI91KeyGenerator;
import cryptix.provider.key.RC2KeyGenerator;
import cryptix.provider.key.RC4KeyGenerator;
import cryptix.provider.key.SAFERKeyGenerator;
import cryptix.provider.key.SPEEDKeyGenerator;
import cryptix.provider.key.SquareKeyGenerator;

import cryptix.provider.rsa.RawRSAPublicKey;
import cryptix.provider.rsa.BaseRSAKeyPairGenerator;
import cryptix.provider.rsa.BaseRSAPublicKey;
import cryptix.provider.elgamal.BaseElGamalKeyPairGenerator;

import java.awt.Cursor;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.FileOutputStream;
import java.security.KeyPairGenerator;
import xjava.security.KeyGenerator;
import java.security.SecureRandom;
import java.util.Enumeration;
import java.util.Vector;
import java.io.Serializable;
import javax.swing.*;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JOptionPane;
import javax.swing.JTree;
import javax.swing.JFrame;
import javax.swing.SwingConstants;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;

import java.net.*;
import java.io.*;

import smi.io.IOStream;
import smi.io.ObjectIOStream;
import smi.mechanisms.Mechanism;
import smi.mechanisms.MechanismInstance;

import ssonet.Preload;
import ssonet.SSONETConstants;

final public class PerformanceTest extends Thread implements Serializable {
    static long[] timeBuffer = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
    static String  symmetricCipherAlgorithms[] = {
	"Blowfish",
	"CAST5",
	"DES",
	"DES3",
	"IDEA",
	"LOKI91",
	"RC2",
	"RC4",
	"SAFER",
	"SPEED",
	"Square"
    };

    static String asymmetricCipherAlgorithms[] = {
	"RSA"
    };

    static String  cipherAlgorithmsForIntegrity[] = {
	"Blowfish",
	"CAST5",
	"DES",
	"DES3",
	"IDEA",
	"LOKI91",
	"RC2",
	"RC4",
	"SAFER",
	"SPEED",
	"Square"
    };

    static String signatureAlgorithms[] = {
	"RSA",
	    "ElGamal",
	    "DSA"
    };

    JLabel statusLabel;
    byte[] buffer;
    JProgressBar progressBar;
    boolean closeTest = false;
    boolean randomInitialized = false;

    public PerformanceTest() {}

    public boolean shutdown() {
	return closeTest || isInterrupted();
    }

    public void run() {
	try {
	    JFrame frame = new JFrame("Performance test");
	    frame.setSize(730,580);
	    JTree jTree = new JTree();
	    jTree.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
	    jTree.setFont(new Font("Monospaced", Font.PLAIN,12));
	    JLabel label = new JLabel("Performance test is running ...", SwingConstants.CENTER);
	    label.setFont(new Font("Dialog", Font.BOLD, 24));
	    frame.getContentPane().add("Center", label);
	    frame.setDefaultCloseOperation (javax.swing.JFrame.DO_NOTHING_ON_CLOSE);
	    frame.addWindowListener(new WindowAdapter() {
		public void windowClosing (WindowEvent e) {
		    if (!randomInitialized) System.out.println("Have to wait until RandomGenerator finish.");
		    closeTest = true;
		}
	    });
	    progressBar = new JProgressBar(0,30);
	    progressBar.setStringPainted (true);

	    JPanel statusPanel = new JPanel();
	    GridBagConstraints gridBagConstraints = new GridBagConstraints();
	    GridBagLayout gridBagLayout = new GridBagLayout();
	    statusPanel.setLayout(gridBagLayout);
	    gridBagConstraints.insets.left = 5;
	    gridBagConstraints.insets.right = 5;
	    gridBagConstraints.insets.top = 3;
	    gridBagConstraints.insets.bottom = 5;
	    gridBagConstraints.gridx = 0;
	    gridBagConstraints.gridy = 0;
	    gridBagConstraints.weightx = 1;
	    statusLabel = new JLabel("dfjkg hdfg usdfhginitialisiere Schlsselgenerator");
	    gridBagLayout.setConstraints(statusLabel, gridBagConstraints);
	    statusPanel.add(statusLabel);
	    gridBagConstraints.gridx = 1;
	    gridBagConstraints.gridy = 0;
	    gridBagConstraints.weightx = 0;
	    gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
	    gridBagLayout.setConstraints(progressBar, gridBagConstraints);
	    statusPanel.add(progressBar);
	    frame.getContentPane().add("South", statusPanel);
	    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	    Dimension ownDimension = frame.getSize();
	    frame.setLocation((screenSize.width - ownDimension.width) / 2,
		(screenSize.height - ownDimension.height) / 2);
	    frame.show();

	    statusLabel.setText("initialize Keygenerator");
	    (new SecureRandom()).nextInt();
	    randomInitialized = true;
	    progressBar.setValue(4);

	    buffer = new byte[64000];
	    try {
		(getClass().getResourceAsStream("data/test.dat")).read(buffer);
	    } catch(Exception e) {
		System.out.println("Error while reading file");
		e.printStackTrace();
		System.exit(0);
	    }
	    DefaultMutableTreeNode rootTreeNode = new DefaultMutableTreeNode("Security goals");
	    DefaultMutableTreeNode confidentialityTreeNode = new DefaultMutableTreeNode("Confidentiality");
	    rootTreeNode.add(confidentialityTreeNode);
	    DefaultMutableTreeNode symmetricConfidentialityTreeNode = new DefaultMutableTreeNode("symmetric methods");
	    confidentialityTreeNode.add(symmetricConfidentialityTreeNode);
	    KeyGenerator symmetricKeyGenerators[] = {
		new BlowfishKeyGenerator(),
		new CAST5KeyGenerator(),
		new DESKeyGenerator(),
		new DES_EDE3KeyGenerator(),
		new IDEAKeyGenerator(),
		new LOKI91KeyGenerator(),
		new RC2KeyGenerator(),
		new RC4KeyGenerator(),
		new SAFERKeyGenerator(),
		new SPEEDKeyGenerator(),
		new SquareKeyGenerator()
	    };

	    if (shutdown()) throw new InterruptedException();
	    //Compiler.disable();
	    testAlgorithms(symmetricCipherAlgorithms,
		null,
		symmetricKeyGenerators,
		symmetricConfidentialityTreeNode,
		SSONETConstants.CONFIDENTIALITY,
		0);
	    jTree.revalidate();
	    System.out.println("Confidentiality mechanism ready");

	    KeyPairGenerator asymmetricCipherKeyGenerators[] = new KeyPairGenerator[1];
	    try {
		asymmetricCipherKeyGenerators[0] = new BaseRSAKeyPairGenerator();
	    } catch(Exception e) {
		e.printStackTrace();
	    }
	    DefaultMutableTreeNode asymmetricConfidentialityTreeNode = new DefaultMutableTreeNode("asymmetric methods");
	    confidentialityTreeNode.add(asymmetricConfidentialityTreeNode);
	    testAlgorithms(asymmetricCipherAlgorithms,
		asymmetricCipherKeyGenerators,
		null,
		asymmetricConfidentialityTreeNode,
		SSONETConstants.CONFIDENTIALITY,
		symmetricCipherAlgorithms.length);

	    DefaultMutableTreeNode integrityTreeNode = new DefaultMutableTreeNode("Integrity");
	    rootTreeNode.add(integrityTreeNode);
	    testAlgorithms(cipherAlgorithmsForIntegrity,
		null,
		symmetricKeyGenerators,
		integrityTreeNode,
		SSONETConstants.INTEGRITY,
		symmetricCipherAlgorithms.length+asymmetricCipherAlgorithms.length);

	    KeyPairGenerator signKeyGenerators[] = new KeyPairGenerator[3];
	    try {
		signKeyGenerators[0] = new BaseRSAKeyPairGenerator();
		signKeyGenerators[1] = new BaseElGamalKeyPairGenerator();
		signKeyGenerators[2] = KeyPairGenerator.getInstance("DSA");
	    } catch(Exception e) {
		e.printStackTrace();
	    }
	    DefaultMutableTreeNode accountabilityTreeNode = new DefaultMutableTreeNode("Accountability");
	    rootTreeNode.add(accountabilityTreeNode);
	    testAlgorithms(signatureAlgorithms,
		signKeyGenerators,
		null,
		accountabilityTreeNode,
		SSONETConstants.ACCOUNTABILITY_ACCEPT,
		symmetricCipherAlgorithms.length+asymmetricCipherAlgorithms.length+cipherAlgorithmsForIntegrity.length);

	    jTree.setModel(new DefaultTreeModel(rootTreeNode));
	    jTree.setCursor(Cursor.getDefaultCursor());
	    jTree.expandRow(0);
	    jTree.expandRow(3);
	    jTree.expandRow(4);
	    jTree.expandRow(2);
	    jTree.expandRow(3);
	    jTree.expandRow(1);
	    jTree.expandRow(3);
	    jTree.expandRow(2);
	    frame.getContentPane().remove(statusPanel);
	    frame.getContentPane().remove(label);
	    frame.getContentPane().add("Center", new JScrollPane(jTree));
	    frame.getContentPane().validate();

	    System.out.println("\n*Confidentiality - symmetric");
	    System.out.println("------------------------------");
	    for(int i = 0; i < symmetricCipherAlgorithms.length; i++)
		System.out.println("Durchsatz " + symmetricCipherAlgorithms[i] + ": " + timeBuffer[i] + "KBytes/s");
	    System.out.println("\n*Confidentiality - asymmetric");
	    System.out.println("------------------------------");
	    for(int i = 0; i < asymmetricCipherAlgorithms.length; i++)
		System.out.println("Durchsatz " + asymmetricCipherAlgorithms[i] + ": " +
		timeBuffer[i + symmetricCipherAlgorithms.length] + "KBytes/s");
	    System.out.println("\n*Integrity - symmetric");
	    System.out.println("------------------------------");
	    for(int i = 0; i < cipherAlgorithmsForIntegrity.length; i++)
		System.out.println("Durchsatz " + cipherAlgorithmsForIntegrity[i] + ": " +
		timeBuffer[i + asymmetricCipherAlgorithms.length + symmetricCipherAlgorithms.length] +
		"KBytes/s");
	    System.out.println("\n*Accountability - asymmetric");
	    System.out.println("------------------------------");
	    for(int i = 0; i < signatureAlgorithms.length; i++)
		System.out.println("Durchsatz " + signatureAlgorithms[i] + ": " +
		timeBuffer[i + asymmetricCipherAlgorithms.length +
		cipherAlgorithmsForIntegrity.length + symmetricCipherAlgorithms.length] +
		"KBytes/s");
	    writeFiles();
	    //Compiler.enable();
	    System.out.println ("Performancetest finished.");
	    frame.setDefaultCloseOperation (javax.swing.JFrame.DISPOSE_ON_CLOSE);
	} catch (InterruptedException ie) {
	    //Compiler.enable();
	    System.out.println ("Performencetest interrupted");
	}
    }

    private static void writeFiles() {
	Vector confidentialityVector = rating.Rating.getAlgValues(System.getProperty("ssonet_cfg")+"rating/confidentiality.dat");
	writeVectorToFile(confidentialityVector, System.getProperty("ssonet_cfg")+"rating/confidentiality.dat",0);

	Vector integrityVector = rating.Rating.getAlgValues(System.getProperty("ssonet_cfg")+"rating/integrity.dat");
	writeVectorToFile(integrityVector, System.getProperty("ssonet_cfg")+"rating/integrity.dat", symmetricCipherAlgorithms.length);

	Vector accountabilityVector = rating.Rating.getAlgValues(System.getProperty("ssonet_cfg")+"rating/accountability.dat");
	writeVectorToFile(accountabilityVector,
	    System.getProperty("ssonet_cfg")+"rating/accountability.dat",
	    asymmetricCipherAlgorithms.length +
	    cipherAlgorithmsForIntegrity.length +
	    symmetricCipherAlgorithms.length);
    }

    private static void writeVectorToFile(Vector vector, String fileName, int i)
    {
	try {
	    FileOutputStream fileOutputStream = new FileOutputStream(fileName);
	    DataOutputStream dataOutputStream = new DataOutputStream(fileOutputStream);
	    for (Enumeration enumeration = vector.elements(); enumeration.hasMoreElements();) {
		rating.AlgorithmDescriptor descriptor = (rating.AlgorithmDescriptor)enumeration.nextElement();
		descriptor.factor4 = (int)timeBuffer[i];
		i++;
		dataOutputStream.writeBytes(
		    "\"" + descriptor.name + "\"" + " " +
		    descriptor.factor1 + " " +
		    descriptor.factor2 + " " +
		    descriptor.factor3 + " " +
		    descriptor.factor4 + " " +
		    descriptor.factor5 + " " +
		    descriptor.factor6 + " " +
		    descriptor.factor7 + "\n"
		    );
	    }
	    System.out.println("File " + fileName + " written");
	    dataOutputStream.close();
	    fileOutputStream.close();
	} catch (Exception e) {
	    System.out.println("Error while writing file: " + fileName);
	    e.printStackTrace();
	}
    }




    private void testAlgorithms (String[] algorithmNames, KeyPairGenerator[] keyPairGenerators,
				 KeyGenerator[] keyGenerators, DefaultMutableTreeNode treeNode,
				 int securityConstant, int timeBufferOffset) throws InterruptedException {

	//Compiler.disable();
	String securityGoal = null;
	String className = "";
	switch(securityConstant) {
	case SSONETConstants.CONFIDENTIALITY:
	    className = "confidentiality.";
	    securityGoal="Confidentiality ";
	    break;
	case SSONETConstants.INTEGRITY:
	    className = "integrity.";
	    securityGoal="Integrity ";
	    break;
	case SSONETConstants.ACCOUNTABILITY_ACCEPT:
	    className = "accountability.";
	    securityGoal="Accountability ";
	}

	for(int i = 0; i < algorithmNames.length; i++) {
	    if (shutdown()) throw new InterruptedException();
	    try {
		ObjectIOStream mechanismIOStream = new ObjectIOStream (null, null);
		Mechanism mechanism = (Mechanism)Class.forName("smi.mechanisms." + className + algorithmNames[i]).newInstance();
  //nderung//////////////////////////////////////////////////////
		mechanism.init(mechanismIOStream, MechanismInstance.getMechanismConfiguration(securityConstant, algorithmNames[i]));
		String algorithmString = algorithmNames[i] + "           +";

		algorithmString = algorithmString.substring(0,9) + "Key creation: ";
		statusLabel.setText(securityGoal + algorithmNames[i] + " - create Key");
		System.out.println(securityGoal + algorithmNames[i] + " - create Key");
		long firstTime = System.currentTimeMillis();
		if(keyPairGenerators == null) {
		    if (keyGenerators[i] != null)
			keyGenerators[i].generateKey();
		} else {
		    if (keyPairGenerators[i] != null) {
			keyPairGenerators[i].initialize (512, new SecureRandom());
			keyPairGenerators[i].generateKeyPair();
		    }
		}
		long secondTime = System.currentTimeMillis();
		String keyCreationTime = Long.toString(secondTime - firstTime);
		algorithmString+=("     ").substring(keyCreationTime.length()) + keyCreationTime + " ms - cipher: ";

		statusLabel.setText(securityGoal + algorithmNames[i] + " - cipher");
		System.out.println(securityGoal + algorithmNames[i] + " - cipher");
		firstTime = System.currentTimeMillis();
		byte[] cryptbuffer = mechanism.testEncrypt (buffer);
		secondTime = System.currentTimeMillis();
		timeBuffer[i+timeBufferOffset] = 64000/(secondTime - firstTime + 1);
		String timeNeededForCipher = Long.toString(secondTime - firstTime);
		algorithmString +=
		    ("     ").substring(timeNeededForCipher.length()) +
		    timeNeededForCipher +
		    " ms - decipher: ";

		statusLabel.setText(securityGoal + algorithmNames[i] + " - decipher");
		System.out.println(securityGoal + algorithmNames[i] + " - decipher");
		System.out.println("decrypt Object now");
		firstTime = System.currentTimeMillis();
		mechanism.testDecrypt(cryptbuffer);
		secondTime = System.currentTimeMillis();
		timeBuffer[i+timeBufferOffset] = (timeBuffer[i+timeBufferOffset] + 64000 / (secondTime - firstTime + 1)) / 2;
		algorithmString += Long.toString(secondTime - firstTime) + " ms";
		treeNode.add(new DefaultMutableTreeNode(algorithmString));
		System.out.println(algorithmString);
		progressBar.setValue(progressBar.getValue() + 1);
	    } catch(Exception e) {
		System.out.println("Error at algorithm: "+algorithmNames[i]);
		e.printStackTrace();
		progressBar.setValue(progressBar.getValue() + 1);
		treeNode.add(new DefaultMutableTreeNode("Error occoured on "+algorithmNames[i]));
	    }
	}
	//Compiler.enable();
    }

    public static void main(String[] argv) {
	PerformanceTest performanceTest = new PerformanceTest();
	performanceTest.start();
	try {
	    performanceTest.join();
	} catch(Exception e) {
	    e.printStackTrace();
	}
    }
}
