/*
 * smi/mechanisms/AbstractAccountabilityMechanism.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package smi.mechanisms;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.ByteArrayInputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.File;
import xjava.security.Cipher;
import xjava.security.Mode;
import java.security.Key;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.PublicKey;
import java.security.PrivateKey;
import java.security.SecureRandom;
import java.security.Signature;
import java.security.SignatureException;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.StringTokenizer;
import java.util.ResourceBundle;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import smi.baseConf.NewCertificateDialog;
import smi.baseConf.BaseConfiguration;
import smi.io.IOStream;
import smi.mechanisms.DetailConfigurationDialog;
import smi.mechanisms.MechanismConfiguration;
import smi.mechanisms.Mechanism;
import smi.mechanisms.ParameterNotCompatibleException;

import ssonet.KeyExchange;

import sun.security.util.BigInt;
import sun.security.util.DerValue;
import sun.security.util.DerOutputStream;
import sun.security.x509.AlgorithmId;
import sun.security.x509.X500Name;
import sun.security.x509.X500Signer;
import sun.security.x509.X509Cert;
import sun.security.x509.X509Key;

public class AbstractAccountabilityMechanism implements IOStream, Mechanism, ActionListener {
    protected Signature signatureIn, signatureOut;
    protected IOStream ioStream;
    protected static String KEY_KEYLENGTH = "KeyLength";
    protected static String PROVIDERLIST = "Providerlist";
    protected String mechanismName;
    protected ResourceBundle resource = BaseConfiguration.getLangResources();
    protected JButton certificateButton = new JButton(resource.getString("REFRESH CERTIFICATE"));
    protected JButton keyButton = new JButton(resource.getString("REFRESH KEY"));
    protected JLabel titleLabel, dateLabel;
    protected JFrame parentFrame;
    protected DetailConfigurationDialog detailConfigurationDialog;
    protected String mechanismCertificateFileName;
    String defaultProvider[] = {"Cryptix"};

    public String[] getModes() {
	return null;
    }

    public int available() {
	return ioStream.available();
    }

    public void close() {
	ioStream.close();
    }

    public MechanismConfiguration getDefaultConfiguration() {
	MechanismConfiguration defaultMechanismConfiguration = new MechanismConfiguration();
	defaultMechanismConfiguration.put(KEY_KEYLENGTH, new Integer(512));
	defaultMechanismConfiguration.put(PROVIDERLIST, defaultProvider);
	return defaultMechanismConfiguration;
    }

    public MechanismConfiguration configDetail(JFrame parentFrame, MechanismConfiguration actualMechanismConfiguration) {
	this.parentFrame = parentFrame;
	detailConfigurationDialog = new DetailConfigurationDialog(parentFrame, mechanismName);

	if(actualMechanismConfiguration == null) actualMechanismConfiguration = getDefaultConfiguration();

	// key length
	Integer keyLength = (Integer)actualMechanismConfiguration.get(KEY_KEYLENGTH);
	if (keyLength == null) keyLength = new Integer(512);
	detailConfigurationDialog.addTabKey(512,1024,8, keyLength.intValue());

	//providers
	detailConfigurationDialog.addTabProvider((String[])actualMechanismConfiguration.get(PROVIDERLIST));

	// certificate...
	detailConfigurationDialog.addTab(resource.getString("CERTIFICATE"), certificateTab());
	detailConfigurationDialog.pack();
	detailConfigurationDialog.setResizable(false);
	detailConfigurationDialog.setLocationRelativeTo(parentFrame);
	detailConfigurationDialog.setVisible(true);
	if(detailConfigurationDialog.getReturnCode()) {
	    actualMechanismConfiguration.put(KEY_KEYLENGTH, new Integer(detailConfigurationDialog.getKeyLength()));
	    actualMechanismConfiguration.put(PROVIDERLIST,
		detailConfigurationDialog.getProviderPreferenceList());
	}
	return actualMechanismConfiguration;
    }

    protected JPanel certificateTab() {
	JPanel certificatePanel = new JPanel();
	certificateButton.addActionListener(this);
	keyButton.addActionListener(this);
	System.out.println("read certificate from "+mechanismCertificateFileName);
	X509Cert mechanismCertificate = new X509Cert();
	mechanismCertificate = KeyExchange.readCertificate(mechanismCertificateFileName);
	if (mechanismCertificate == null) {
	    titleLabel = new JLabel (resource.getString("NO CERTIFICATE PRESENT"), JLabel.CENTER);
	    dateLabel = new JLabel(resource.getString("PLEASE CREATE A CERTIFICATE"), JLabel.CENTER);
	} else {
	    titleLabel = new JLabel (resource.getString("THIS CERTIFICATE IS VALID TO"), JLabel.CENTER);
	    dateLabel = new JLabel(mechanismCertificate.getNotAfter().toString(), JLabel.CENTER);
	}
	GridBagLayout gridBagLayout = new GridBagLayout();
	GridBagConstraints gridBagConstraints = new GridBagConstraints();
	certificatePanel.setLayout(gridBagLayout);

	gridBagConstraints.insets.top = 10;
	gridBagConstraints.gridx = 0;
	gridBagConstraints.gridy = 0;
	gridBagConstraints.gridwidth = 1;
	gridBagConstraints.gridheight = 1;
	gridBagConstraints.weightx = 0;
	gridBagConstraints.weighty = .25;
	gridBagLayout.setConstraints(titleLabel, gridBagConstraints);
	certificatePanel.add(titleLabel);

	gridBagConstraints.insets.top = 0;
	gridBagConstraints.gridy = 1;
	gridBagConstraints.weighty = .25;
	gridBagLayout.setConstraints(dateLabel, gridBagConstraints);
	certificatePanel.add(dateLabel);

	gridBagConstraints.gridy = 2;
	gridBagLayout.setConstraints(certificateButton, gridBagConstraints);
	certificatePanel.add(certificateButton);

	gridBagConstraints.gridy = 3;
	gridBagConstraints.insets.bottom = 10;
	gridBagLayout.setConstraints(keyButton, gridBagConstraints);
	certificatePanel.add(keyButton);
	return certificatePanel;
    }

    public void actionPerformed(ActionEvent actionEvent) {
	Object object = actionEvent.getSource();
	if(object == certificateButton) {
	    createNewCertificate(readMechanismPublicKey());
	}
	else if(object == keyButton) {
	    createKeyPair();
	    createNewCertificate(readMechanismPublicKey());
	}
    }

    protected X509Key readMechanismPublicKey() {
	return null;
    }

    protected void writeMechanismPublicKey(PublicKey publicKey) {
    }

    protected void writeMechanismPrivateKey(PrivateKey privateKey) {
    }

    protected X509Key createX509Key(String algorithmID, byte[] rawKeyData) {
	try {
	    // add leading zero
	    byte[] rawKeyData2 = new byte[rawKeyData.length+1];
	    rawKeyData2[0] = 0;
	    for (int i = 0; i < rawKeyData.length; i++) rawKeyData2[i + 1] = rawKeyData[i];

	    byte[] algIDArray = smi.util.Emit.emit(algorithmID);

	    DerValue algDerValue = new DerValue(algIDArray);
	    DerValue keyDerValue = new DerValue(DerValue.tag_BitString, rawKeyData2);

	    DerValue[] both = {algDerValue, keyDerValue};
	    DerOutputStream derOutputStream2 = new DerOutputStream();
	    derOutputStream2.putSequence(both);
	    byte[] bothArray = derOutputStream2.toByteArray();
	    DerValue bothDerValue = new DerValue(bothArray);

	    return (X509Key) X509Key.parse(bothDerValue);
	} catch(Exception e) {
	    System.out.println("Error in AbstractAccountabilityMechanism.createX509Key");
	    e.printStackTrace();
	}
	return null;
    }

    protected void createNewCertificate(X509Key publicKey) {
	try {
	    System.out.println("reading sign key");
	    FileInputStream fileInputStream = new FileInputStream(System.getProperty("ssonet_cfg")
		+"users/"+System.getProperty("ssonet_userdir") +"/KeyExchangeDSAPrivateKey.object");
	    ObjectInputStream objectInputStream = new ObjectInputStream(fileInputStream);
	    PrivateKey privateKey = (PrivateKey)objectInputStream.readObject();
	    objectInputStream.close();
	    fileInputStream.close();

	    System.out.println("reading sign certificate");
	    fileInputStream = new FileInputStream(System.getProperty("ssonet_cfg","")
		+"users/"+System.getProperty("ssonet_userdir") +"/KeyExchangeDSACertificate.der");
	    X509Cert issuerCertificate = new X509Cert();
	    issuerCertificate.decode(fileInputStream);
	    fileInputStream.close();

	    X500Signer signer = issuerCertificate.getSigner(new AlgorithmId(AlgorithmId.DSA_oid), privateKey);
	    X500Name subjectName = issuerCertificate.getSubjectName();

	    GregorianCalendar gregorianCalendar = (GregorianCalendar)Calendar.getInstance();
	    Date notBefore = gregorianCalendar.getTime();

	    int duration = new Integer(NewCertificateDialog.getAnswer(parentFrame)).intValue();
	    gregorianCalendar.add(Calendar.DAY_OF_YEAR, duration);
	    Date notAfter = gregorianCalendar.getTime();

	    X509Cert certificate = new X509Cert(subjectName, publicKey, notBefore, notAfter);

	    System.out.println("sign certificate");
	    SecureRandom randomGenerator = new SecureRandom();
	    byte[] randomBuffer = new byte[16];
	    randomGenerator.nextBytes(randomBuffer);
	    X509Cert signedCertificate = new X509Cert(certificate.encodeAndSign(new BigInt(randomBuffer), signer));
	    System.out.println("Certifikat:\n"+signedCertificate.toString());

	    System.out.println("writing Certificate");
	    FileOutputStream fileOutputStream = new FileOutputStream(mechanismCertificateFileName);
	    certificate.encode(fileOutputStream);
	    fileOutputStream.close();

	    titleLabel.setText(resource.getString("THIS CERTIFICATE IS VALID TO"));
	    dateLabel.setText(signedCertificate.getNotAfter().toString());
	} catch(Exception ex) {
	    System.out.println(ex);
	}
    }

    protected void createKeyPair() {
	try {
	    KeyPairGenerator keyPairGenerator = KeyPairGenerator.getInstance(mechanismName);
	    int keyLength = detailConfigurationDialog.getKeyLength();
	    System.out.println("initialize KeyPairGenerator for "+keyLength+" Bit KeyPair");
	    keyPairGenerator.initialize(keyLength);
	    System.out.println("generate KeyPair.");
	    KeyPair ownKeyPair = keyPairGenerator.generateKeyPair();

	    System.out.println("public key:\n"+ownKeyPair.getPublic().toString());
	    System.out.println("\nprivate key:\n"+ownKeyPair.getPrivate().toString());

	    writeMechanismPublicKey(ownKeyPair.getPublic());
	    writeMechanismPrivateKey(ownKeyPair.getPrivate());
	} catch(Exception e) {
	    System.out.println("Error in AbstractAccountabilityMechanism.createKeyPair");
	    e.printStackTrace();
	}
    }

    /**
     * Create directory for accountability keys, if it does not exist and check writability
     */
    protected void createKeyDirectory() throws Exception{
	File acDir = new File(System.getProperty("ssonet_cfg","")
	    +"users/"+System.getProperty("ssonet_userdir")+"/accountability");
	if(!acDir.exists()){
	    acDir.mkdir();
	}
	if(!acDir.canWrite()) throw new Exception("Directory for keys is not writable.");
    }

    /** Initialize without key exchange
    * @param ioStream mechanism IOStream
    * @param ownMechanismConfiguration own mechanism configuration
    */
    public void init(IOStream ioStream,
	MechanismConfiguration ownMechanismConfiguration) {
    }

    /** Initialize with key exchange
    * @param ioStream mechanism IOStream
    * @param ownMechanismConfiguration own mechanism configuration
    * @param partnerMechanismConfiguration partner mechanism configuration
    * @param keyExchange prepared key exchange protocol
    * @exception Exception if initialization fails
    */
    public void init(IOStream ioStream,
	MechanismConfiguration ownMechanismConfiguration,
	MechanismConfiguration partnerMechanismConfiguration,
	KeyExchange keyExchange) throws Exception {
    }

    public byte[] testEncrypt(byte[] buffer)
    {
	try {
	    SignatureObject signatureObject = new SignatureObject(buffer);
	    signatureOut.update(signatureObject.getObjectAsByteArray());
	    return signatureObject.getObjectAsByteArray();
	} catch (SignatureException se) {
	    System.out.println ("SignatureException in Accountability-encrypt");
	    se.printStackTrace();
	    return null;
	}
    }

    public byte[] testDecrypt(byte[] buffer)
    {
	try {
	    SignatureObject signatureObject = new SignatureObject(buffer);
	    signatureIn.update(signatureObject.getObjectAsByteArray());
	    return signatureObject.getObjectAsByteArray();
	} catch (SignatureException se) {
	    System.out.println ("SignatureException in Accountability-decrypt");
	    se.printStackTrace();
	    return null;
	}
    }

    public Object readObject() throws Exception {
	try {
	    SignatureObject signatureObject = (SignatureObject)ioStream.readObject();
	    signatureIn.update(signatureObject.getObjectAsByteArray());
	    if(signatureIn.verify(signatureObject.getSignature()))
	    {
		return signatureObject.getObject();
	    }
	    throw new Exception();
	} catch (Exception e) {
	    throw e;
	}
    }

    public void writeObject(Object object) throws Exception {
	try {
	    SignatureObject signatureObject = new SignatureObject(object);
	    signatureOut.update(signatureObject.getObjectAsByteArray());
	    signatureObject.setSignature(signatureOut.sign());
	    ioStream.writeObject(signatureObject);
	} catch (Exception e) {
	    throw e;
	}
    }
}
