package smi.mechanisms;

import smi.baseConf.BaseConfiguration;
import smi.io.IOStream;
import smi.mechanisms.DetailConfigurationDialog;
import smi.mechanisms.MechanismConfiguration;
import smi.mechanisms.Mechanism;
import smi.mechanisms.ParameterNotCompatibleException;

import ssonet.KeyExchange;
import ssonet.SSONETConnection;
import ssonet.SSONETConstants;

import java.util.StringTokenizer;
import java.security.SecureRandom;

import javax.swing.JFrame;


/**
 * This class provides abstract anonymity. As this class also 
 * implements an IOStream this IOStream must be the last one in
 * a row, because this Stream is also used to connect the client 
 * to the server.
 */
public class AbstractAnonymityMechanism implements IOStream, Mechanism
{
    protected String mechanismName;
    protected IOStream ioStream;
    
    /** returns the availability of ioStream */
    public int available() {
		return ioStream.available();
    }
	
    /** closes ioStream */
    public void close() {
		ioStream.close();
    }
	
    /** reads an object from ioStream */
    public Object readObject() throws Exception {
    	return null;
    }
	
    /** writes an object to ioStream */
    public void writeObject(Object o) throws Exception {
		
    }
	
    /** Getting the dafault Mechanism Configuration */
    public MechanismConfiguration getDefaultConfiguration() {
    	MechanismConfiguration defaultMechanismConfiguration = new MechanismConfiguration();
    	// change the default configuration here.
    	return defaultMechanismConfiguration;
    }
	
    /** detailed configuration of anonymity features */
    public MechanismConfiguration configDetail(JFrame parentFrame,
                                               MechanismConfiguration actualMechanismConfiguration) {
        return null;
    }
	

    /** Initialize with key exchange
    * @param ioStream mechanism IOStream
    * @param ownMechanismConfiguration own mechanism configuration
    * @param partnerMechanismConfiguration partner mechanism configuration
    * @param keyExchange prepared key exchange protocol
    * @exception Exception if initialization fails
    */
    public void init(IOStream ioStream,
                     MechanismConfiguration ownMechanismConfiguration,
                     MechanismConfiguration partnerMechanismConfiguration,
                     KeyExchange keyExchange) throws Exception {
    }

    
    /** not used, just here to meet requirements in @see Mechanism */
    public void init(IOStream ioStream,
                     MechanismConfiguration ownMechanismConfiguration) {
    }
    
    /** not used, just here to meet requirements in @see Mechanism */
	public byte[] testEncrypt(byte[] buffer) {
		return null;
	}
    
    /** not used, just here to meet requirements in @see Mechanism */
	public byte[] testDecrypt(byte[] buffer) {
		return null;
	}
    
    /** not used, just here to meet requirements in @see Mechanism */
    public String[] getModes() {
    	return null;
    }
}
