/*
 * experimental source code
 * smi/mechanisms/accountability/ElGamal.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package smi.mechanisms.accountability;

import cryptix.provider.elgamal.BaseElGamalPublicKey;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.math.BigInteger;
import java.security.Signature;
import java.security.KeyPair;
import java.security.PublicKey;
import java.security.PrivateKey;
import java.security.SecureRandom;

import javax.swing.JFrame;
import javax.swing.JPanel;

import smi.awt.JPreferenceList;
import smi.baseConf.RatedString;
import smi.io.IOStream;
import smi.mechanisms.AbstractAccountabilityMechanism;
import smi.mechanisms.DetailConfigurationDialog;
import smi.mechanisms.MechanismConfiguration;
import smi.mechanisms.Mechanism;
import smi.mechanisms.ParameterNotCompatibleException;
import smi.baseConf.BaseConfiguration;
import ssonet.KeyExchange;
import ssonet.SSONETConnection;

import sun.security.x509.X509Cert;
import sun.security.x509.X509Key;

public class ElGamal extends AbstractAccountabilityMechanism {
    private JPreferenceList variantPreferenceList;
    private static String variants[] = {"MD-2","MD-5","RIPEMD-160","SHA-1"};
    private static String KEY_VARIANT = "KeyVariante";

    public ElGamal() {
	mechanismName = "ElGamal";
	mechanismCertificateFileName = new String(System.getProperty("ssonet_cfg","")
	+"users/"+System.getProperty("ssonet_userdir")+"/accountability/ElGamalCertificate.der");
	System.out.println(mechanismName + " Sign instance created");
    }

    public MechanismConfiguration getDefaultConfiguration() {
	MechanismConfiguration  defaultMechanismConfiguration = new MechanismConfiguration();
	defaultMechanismConfiguration.put(KEY_KEYLENGTH, new Integer(512));
	String[] defaultMode = {"SHA-1"};
	String defaultProvider[] = {"Cryptix"};
	defaultMechanismConfiguration.put(PROVIDERLIST, defaultProvider);
	defaultMechanismConfiguration.put(KEY_VARIANT, defaultMode);
	return defaultMechanismConfiguration;
    }

    public MechanismConfiguration configDetail(JFrame parentFrame, MechanismConfiguration actualConfiguration) {
	detailConfigurationDialog = new DetailConfigurationDialog(parentFrame,"ElGamal");
	this.parentFrame = parentFrame;
	if(actualConfiguration == null) actualConfiguration = getDefaultConfiguration();

	// providers
	detailConfigurationDialog.addTabProvider((String[])actualConfiguration.get(PROVIDERLIST));

	// key length
	Integer akt = (Integer)actualConfiguration.get(KEY_KEYLENGTH);
	if (akt == null) akt = new Integer(512);
	detailConfigurationDialog.addTabKey(512,1024,8, akt.intValue());

	// variants...
	detailConfigurationDialog.addTab(BaseConfiguration.getLangResources().getString("VARIANTS"), variantTab((String[])actualConfiguration.get(KEY_VARIANT)));

	// certificate
	detailConfigurationDialog.addTab(BaseConfiguration.getLangResources().getString("CERTIFICATE"), certificateTab());

	detailConfigurationDialog.pack();
	detailConfigurationDialog.setResizable(false);
	detailConfigurationDialog.setLocationRelativeTo(parentFrame);
	detailConfigurationDialog.setVisible(true);
	if(detailConfigurationDialog.getReturnCode()) {
	    actualConfiguration.put(PROVIDERLIST, detailConfigurationDialog.getProviderPreferenceList());
	    actualConfiguration.put(KEY_KEYLENGTH, new Integer(detailConfigurationDialog.getKeyLength()));
	    actualConfiguration.put(KEY_VARIANT, variantPreferenceList.getPreferenceListItems());
	}
	return actualConfiguration;
    }

    private byte[] int2byte (int input)
    {
	byte[] output = new byte[2];
	output[1] = (byte)(output.length&0x00FF);
	output[0] = (byte)((output.length>>8)&0xFF00);
	return output;
    }

    protected X509Key readMechanismPublicKey() {
	X509Key publicKey = null;
	try {
	    FileInputStream fileInputStream = new FileInputStream(System.getProperty("ssonet_cfg","")
	    +"users/"+System.getProperty("ssonet_userdir")+"/accountability/ElGamalPublicKey.object");
	    ObjectInputStream objectInputStream = new ObjectInputStream(fileInputStream);
	    BaseElGamalPublicKey baseElGamalPublicKey = (BaseElGamalPublicKey)objectInputStream.readObject();
	    fileInputStream.close();
	    objectInputStream.close();

	    byte[] gArray = baseElGamalPublicKey.getG().toByteArray();
	    byte[] gLength = int2byte (gArray.length);
	    byte[] pArray = baseElGamalPublicKey.getP().toByteArray();
	    byte[] pLength = int2byte (pArray.length);
	    byte[] yArray = baseElGamalPublicKey.getY().toByteArray();
	    byte[] yLength = int2byte (yArray.length);
	    byte[] rawKeyData = new byte[6 + gArray.length + pArray.length + yArray.length];

	    rawKeyData[0] = gLength[0];
	    rawKeyData[1] = gLength[1];
	    int i = 2;
	    for (int j = 0; j<gArray.length; j++) {
		rawKeyData[i] = gArray[j];
		i++;
	    }
	    rawKeyData[i] = pLength[0];
	    i++;
	    rawKeyData[i] = pLength[1];
	    i++;
	    for (int j = 0; j<pArray.length; j++) {
		rawKeyData[i] = pArray[j];
		i++;
	    }
	    rawKeyData[i] = yLength[0];
	    i++;
	    rawKeyData[i] = yLength[1];
	    i++;
	    for (int j = 0; j<yArray.length; j++) {
		rawKeyData[i] = yArray[j];
		i++;
	    }
	    // ElGamal keys are issued more general as Diffie-Hellmann keys
	    // Reason: Sun classes dont know ElGamal and I dont know ElGamals OID
	    publicKey = createX509Key("DH", rawKeyData);
	} catch(Exception e) {
	    System.out.println("Error in ElGamal.readMechanismPublicKey");
	    e.printStackTrace();
	}
	return publicKey;
    }

    protected void writeMechanismPublicKey(PublicKey publicKey) {
	try {
	    createKeyDirectory();
	    FileOutputStream fileOutputStream = new FileOutputStream(System.getProperty("ssonet_cfg","")
	    +"users/"+System.getProperty("ssonet_userdir")+"/accountability/ElGamalPublicKey.object");
	    System.out.println("writing public key to accountability/ElGamalPublicKey.object");
	    ObjectOutputStream objectOutputStream = new ObjectOutputStream(fileOutputStream);
	    objectOutputStream.writeObject(publicKey);
	    fileOutputStream.close();
	    objectOutputStream.close();
	} catch(Exception e) {
	    System.out.println("Error in ElGamal.writeMechanismPublicKey");
	    e.printStackTrace();
	}
    }

    protected void writeMechanismPrivateKey(PrivateKey privateKey) {
	try {
	    createKeyDirectory();
	    FileOutputStream fileOutputStream = new FileOutputStream(System.getProperty("ssonet_cfg","")
	    +"users/"+System.getProperty("ssonet_userdir")+"/accountability/ElGamalPrivateKey.object");
	    System.out.println("writing private key to accountability/ElGamalPrivateKey.object");
	    ObjectOutputStream objectOutputStream = new ObjectOutputStream(fileOutputStream);
	    objectOutputStream.writeObject(privateKey);
	    fileOutputStream.close();
	    objectOutputStream.close();
	} catch(Exception e) {
	    System.out.println("Error in ElGamal.writeMechanismPrivateKey");
	    e.printStackTrace();
	}
    }

    private JPanel variantTab(String[] chosenVariants) {
	JPanel variantPanel = new JPanel();
	variantPreferenceList = new JPreferenceList(5,
	    BaseConfiguration.getLangResources().getString("PREFERENCE LIST"),
	    BaseConfiguration.getLangResources().getString("AVAILABLE VARIANTS"),
	    chosenVariants,
	    variants);
	variantPanel.add(variantPreferenceList);
	return variantPanel;
    }

    /** Initialize without key exchange
    * @param ioStream mechanism IOStream
    * @param ownMechanismConfiguration own mechanism configuration
    */
    public void init(IOStream ioStream,
	MechanismConfiguration ownMechanismConfiguration) {
	try {

	    String[] ownVariants = (String[])ownMechanismConfiguration.get(KEY_VARIANT);
	    String variant = ownVariants[0];
	    System.out.println("Using variant: " + variant);

	    X509Cert certificate = KeyExchange.readCertificate(System.getProperty("ssonet_cfg","")
	    +"users/"+System.getProperty("ssonet_userdir")+"/accountability/ElGamalCertificate.der");
	    PublicKey testKey = certificate.getPublicKey();

	    initCipher(testKey, variant, ioStream);
	}
	catch (Exception e) {
	    System.out.println("Error in ElGamal.init");
	    e.printStackTrace();
	}
    }

    /** Initialize with key exchange
    * @param ioStream mechanism IOStream
    * @param ownMechanismConfiguration own mechanism configuration
    * @param partnerMechanismConfiguration partner mechanism configuration
    * @param keyExchange prepared key exchange protocol
    * @exception Exception if initialization fails
    */
    public void init(IOStream ioStream,
	MechanismConfiguration ownMechanismConfiguration,
	MechanismConfiguration partnerMechanismConfiguration,
	KeyExchange keyExchange) throws Exception {
	try {
	    String[] ownVariants = (String[])ownMechanismConfiguration.get(KEY_VARIANT);
	    String[] partnerVariants = (String[])partnerMechanismConfiguration.get(KEY_VARIANT);
	    String variant = (SSONETConnection.computePreferred(RatedString.toRatedString(ownVariants),
		RatedString.toRatedString(partnerVariants)))[0];
	    System.out.println("Using variant: " + variant);

	    X509Cert mechanismCertificate = keyExchange.readCertificate(System.getProperty("ssonet_cfg","")
	    +"users/"+System.getProperty("ssonet_userdir")+"/accountability/ElGamalCertificate.der");
	    PublicKey testKey = keyExchange.getPartnerMechanismKey(mechanismCertificate);

	    initCipher(testKey, variant, ioStream);
	}
	catch (Exception e) {
	    System.out.println("Error in ElGamal.init");
	    e.printStackTrace();
	    throw new ParameterNotCompatibleException();
	}
    }

    /** Initialize crypto mechanisms
    * @param testKey test key
    * @param variant used hash algorithm
    * @param ioStream IOStream
    */
    private void initCipher(PublicKey testKey, String variant, IOStream ioStream) {
	try {
	    this.ioStream = ioStream;

	    byte[] rawBuffer;
	    if (testKey instanceof BaseElGamalPublicKey)
	    {
		rawBuffer = new byte[204]; // 204 = length of ElGamal Public Key
		SecureRandom sr = new SecureRandom();
		sr.nextBytes (rawBuffer);
	    } else {
		rawBuffer = KeyExchange.DERtoRAW(testKey.getEncoded());
	    }

	    // change raw keybits into ElGamal key
	    int offset = 0;
	    int gLength = rawBuffer[offset] * 256 + rawBuffer[offset + 1];
	    byte[] gArray = new byte[gLength];
	    offset += 2;
	    int i = 0;
	    int end = offset + gLength;
	    while (offset<end) {
		gArray[i] = rawBuffer[offset];
		i++;
		offset++;
	    }

	    int pLength = rawBuffer[offset] * 256 + rawBuffer[offset + 1];
	    byte[] pArray = new byte[pLength];
	    offset += 2;
	    i = 0;
	    end = offset + pLength;
	    while (offset<end) {
		pArray[i] = rawBuffer[offset];
		i++;
		offset++;
	    }

	    int yLength = rawBuffer[offset] * 256 + rawBuffer[offset + 1];
	    byte[] yArray = new byte[yLength];
	    offset += 2;
	    i = 0;
	    end = offset + yLength;
	    while (offset<end) {
		yArray[i] = rawBuffer[offset];
		i++;
		offset++;
	    }

	    BigInteger Gvalue = new BigInteger(gArray);
	    BigInteger Pvalue = new BigInteger(pArray);
	    BigInteger Yvalue = new BigInteger(yArray);

	    BaseElGamalPublicKey baseElGamalPublicKey = new BaseElGamalPublicKey(Pvalue, Gvalue, Yvalue);

	    FileInputStream fileInputStream = new FileInputStream(System.getProperty("ssonet_cfg","")
	    +"users/"+System.getProperty("ssonet_userdir")+"/accountability/ElGamalPrivateKey.object");
	    ObjectInputStream objectInputStream = new ObjectInputStream(fileInputStream);
	    PrivateKey signKey = (PrivateKey)objectInputStream.readObject();

	    signatureIn = Signature.getInstance(variant + "/" + mechanismName);
	    signatureOut = Signature.getInstance(variant + "/" + mechanismName);

	    signatureIn.initVerify(baseElGamalPublicKey);
	    signatureOut.initSign(signKey);
	    System.out.println(mechanismName + " instance inited");
	}
	catch(Exception e) {
	    System.out.println("Error in ElGamal.initCipher");
	    e.printStackTrace();
	}
    }
}
