/*
 * smi/mechanisms/confidentiality/Gateway.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package smi.mechanisms.confidentiality;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import xjava.security.Cipher;
import java.security.Key;
import xjava.security.KeyGenerator;
import java.security.SecureRandom;
import java.util.StringTokenizer;
import java.util.Hashtable;
import java.net.Socket;
import javax.swing.JFrame;
import java.io.*;
import java.net.UnknownHostException;
import javax.swing.JFrame;

import smi.baseConf.RatedString;
import smi.io.*;
import smi.mechanisms.*;
import gateway.GatewayWindow;

import ssonet.KeyExchange;
import ssonet.SSONETConnection;
import ssonet.SSONETConstants;
import java.security.PublicKey;

import cryptix.provider.key.RawSecretKey;
import cryptix.provider.rsa.RawRSAPrivateKey;
import cryptix.provider.rsa.RawRSAPublicKey;

import sun.security.x509.X509Cert;

public class Gateway implements IOStream, Mechanism {
    protected IOStream ioStream, GWioStream;
    protected Cipher CipherIn, CipherOut;
    protected static String PROVIDERLIST = "Providerlist";
    protected static String KEY_MODE_PREFERENCELIST = "Mode.Preferencelist";
    protected static String KEY_KEYLENGTH = "KeyLength";
    protected static String KEY_ROUNDS = "Rounds";
    protected static String KEY_VARIANT = "Variant";
    protected String mechanismName;
    String defaultProvider[] = {"Cryptix"};
    protected X509Cert ownTestCertificate, ownCipherCertificate;
    protected PublicKey trustCenterTestKey;

    private X509Cert mechanismCertificate;
    private String packageName;
    private static String remoteHost;
    private static int remotePort;
    Socket socket;
    KeyExchange GWKeyExchange;
    boolean isServer = false;
    public String[] getModes() {
	String[] modes = {"CBC","CFB","ECB","OFB","PCBC"};
	return modes;
    }

    public int available() {
	return GWioStream.available();
    }

    public void close() {
	GWioStream.close();
    }

    /**
    * Encrypt buffer and return the encrypted data
    * @param buffer  date to crypt
    * @return  encrypted buffer data
    */
    public byte[] testEncrypt (byte[] buffer) {
	return CipherOut.crypt(buffer);
    }

    /**
    * Decrypt buffer and return the decrypted data
    * @param buffer  date to crypt
    * @return  decrypted buffer data
    */
    public byte[] testDecrypt (byte[] buffer) {
	return CipherIn.crypt(buffer);
    }

    public Object readObject() throws Exception {
	try {
	    System.out.println("Lese Objekt von Server");
	    Object object = ioStream.readObject(); //Object vom Server lesen
	    System.out.println("Serverobject angekommen");
	    Hashtable ht = new Hashtable();
	    ht.put("OBJECT", object);
	    ht.put("TODO", new String("DECRYPT&ENCRYPT"));
	    ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
	    ObjectOutputStream objectOutputStream = new ObjectOutputStream(byteArrayOutputStream);
	    objectOutputStream.writeObject(ht);
	    byte[] buffer = byteArrayOutputStream.toByteArray();

	    // verschlsselt zum Gateway bertragen!
	    buffer = CipherOut.crypt(buffer);

	    GWioStream.writeObject(buffer);
	    objectOutputStream.close();
	    byteArrayOutputStream.close();

	    //mit Rckmech. verschl. Object vom Gateway empfangen
	    object = GWioStream.readObject(); //umcodiertes Objekt vom Gateway empfangen

	    //entschlsseln und im Klartext zum Clienten
	    byte[] b = (byte[])object;
	    System.out.println("got recoded obj: "+b[0]+" lgt.: "+b.length);
	    b = CipherIn.crypt(b);

	    int len = new String(b).length();
	    if (len>50)
		len = 50;
	    System.out.println("decrypted obj.: "+ new String(b, 0, len)+" lgt.: "+b.length);
	    System.out.println("Gateway - Classname:" + (String)(object.getClass().getName()));

	    Object returnObject = (new ObjectInputStream(new ByteArrayInputStream(b))).readObject();
	    System.out.println("Gateway readObject, Classname (nach Umcodierung):" + (String)(returnObject.getClass().getName()));

	    return returnObject;

	} catch (Exception e) {
	    System.out.println(e);
	    e.printStackTrace();
	    throw e;
	}
    }

    public void writeObject(Object object) throws Exception {
	try {
	    System.out.println("Gateway: Schreibe Object vom Client zum Gateway");
	    System.out.println("object is of type "+object.getClass().getName());
	    Hashtable ht = new Hashtable();
	    ht.put("OBJECT", object);
	    ht.put("TODO", new String("ENCRYPT"));
	    ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
	    ObjectOutputStream objectOutputStream = new ObjectOutputStream(byteArrayOutputStream);
	    objectOutputStream.writeObject(ht);
	    objectOutputStream.flush();
	    byte[] buffer = byteArrayOutputStream.toByteArray();
	    buffer = CipherOut.crypt(buffer);

	    GWioStream.writeObject(buffer);
	    objectOutputStream.close();
	    byteArrayOutputStream.close();
	    object = GWioStream.readObject();

	    System.out.println("Object zum Server mit HM verschlsselt.");
	    ioStream.writeObject(object);
	} catch (Exception e) {
	    System.out.println("Error at writeObject");
	    System.out.println(e);
	    throw e;
	}
    }


    /** Initialize without key exchange
    * @param ioStream mechanism IOStream
    * @param ownMechanismConfiguration own mechanism configuration
    */
    public void init(IOStream ioStream,
	MechanismConfiguration ownMechanismConfiguration) {
	try {
	    this.ioStream = ioStream;
	    String[] modePreferenceList = (String[])ownMechanismConfiguration.get(KEY_MODE_PREFERENCELIST);
	    String mode = modePreferenceList[0];
	    System.out.println("Using mode: " + mode);

	    StringTokenizer stringTokenizer = new StringTokenizer(getClass().getName(),".");
	    for (int i = 0; i<4; i++) {
		mechanismName = stringTokenizer.nextToken();
	    }

	    System.out.println("generating " + mechanismName + " Key.");
	    SecureRandom random = new SecureRandom();
	    KeyGenerator keyGenerator = KeyGenerator.getInstance(mechanismName);
	    keyGenerator.initialize(random);
	    Key sessionKey = keyGenerator.generateKey();

	    System.out.println("initialize Cipher.");
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }


    /** Initialize with key exchange
    * @param ioStream mechanism IOStream
    * @param ownMechanismConfiguration own mechanism configuration
    * @param partnerMechanismConfiguration partner mechanism configuration
    * @param keyExchange prepared key exchange protocol
    * @exception Exception if initialization fails
    */


    public void init(IOStream ioStream,
	MechanismConfiguration mechs,
	MechanismConfiguration gatewayConfiguration,
	KeyExchange keyExchange) throws Exception {
	try {
	    MechanismConfiguration backMech = (MechanismConfiguration)mechs.get("BACK");
	    MechanismConfiguration hinMech  = (MechanismConfiguration)mechs.get("HIN");
	    this.ioStream = ioStream;
	    remoteHost = ((String[])gatewayConfiguration.get("GATEWAY"))[GatewayWindow.GW_IP];
	    remotePort = Integer.parseInt(((String[])gatewayConfiguration.get("GATEWAY"))[GatewayWindow.GW_PORT]);
	    packageName = (String)gatewayConfiguration.get("PACKAGENAME");
	    System.out.println("....AGW: Try to get sessionKeyHin");
	    Key sessionKeyHin = keyExchange.getSessionKey(((String[])gatewayConfiguration.get("GATEWAY"))[GatewayWindow.GW_HIN_MECH]);
	    try {
		System.out.println("IP: "+remoteHost);
		System.out.println("Port: "+(remotePort));
		socket = new Socket(remoteHost, (remotePort));
	    }
	    catch (UnknownHostException e) {
		System.err.println("Don't know about host: " + remoteHost);
	    }
	    catch (IOException e) {
		System.err.println("Couldn't get I/O for the connection to: " + remoteHost);
	    }

	    InputStream in = socket.getInputStream();
	    OutputStream out = socket.getOutputStream();
	    System.out.println("Gateway erzeugt IOStream");
	    GWioStream = new ObjectIOStream(in, out);

	    System.out.println("GWioStream steht ... Versuche new KeyExchange");
	    X509Cert trustCenterCertificate = KeyExchange.readCertificate(System.getProperty("ssonet_cfg","")
	    +"trustcenter/TrustCenterCertificate.der");
	    trustCenterTestKey = trustCenterCertificate.getPublicKey();
	    System.out.println("testing trustcenter certificate.");
	    if (!KeyExchange.testCertificate(trustCenterCertificate, trustCenterTestKey))
		throw new SecurityException();

	    System.out.println("reading ownTestCertificate:");
	    ownTestCertificate = KeyExchange.readCertificate(System.getProperty("ssonet_cfg","")
	    +"users/"+System.getProperty("ssonet_userdir")+"/KeyExchangeDSACertificate.der");
	    PublicKey ownPublicKey = ownTestCertificate.getPublicKey();

	    System.out.println("testing ownTestCertificate.");
	    if (!KeyExchange.testCertificate(ownTestCertificate, trustCenterTestKey))
		throw new SecurityException();

	    System.out.println("reading ownCipherCertificate:");
	    ownCipherCertificate = KeyExchange.readCertificate(System.getProperty("ssonet_cfg","")
	    +"users/"+System.getProperty("ssonet_userdir")+"/KeyExchangeRSACertificate.der");

	    System.out.println("testing ownCipherCertificate.");
	    if (!KeyExchange.testCertificate(ownCipherCertificate, ownPublicKey))
		throw new SecurityException();

	    GWKeyExchange = new KeyExchange(GWioStream,
		null,
		ownTestCertificate,
		ownCipherCertificate,
		trustCenterTestKey,
		isServer);

	    mechanismCertificate = GWKeyExchange.readCertificate(System.getProperty("ssonet_cfg","")
	    +"users/"+System.getProperty("ssonet_userdir")+"/confidentiality/RSACertificate.der");
	    PublicKey publicKey = GWKeyExchange.getPartnerMechanismKey(mechanismCertificate);
	    System.out.println("publicKey: "+publicKey);

	    GWioStream.writeObject(mechs);
	    GWioStream.writeObject(((String[])gatewayConfiguration.get("GATEWAY"))[GatewayWindow.GW_BACK_MECH]);
	    GWioStream.writeObject(packageName);
	    GWioStream.writeObject(sessionKeyHin);

	    Key sessionKey = GWKeyExchange.getSessionKey(((String[])gatewayConfiguration.get("GATEWAY"))[GatewayWindow.GW_BACK_MECH]);

	    System.out.println("sessionKey(RM): "+sessionKey+"\nAlg:  "+sessionKey.getAlgorithm());
	    System.out.println("measurement running with Gateway");
	    String mode = ((String[])backMech.get(KEY_MODE_PREFERENCELIST))[0];
	    initCipher(sessionKey.getAlgorithm(), sessionKey, mode);
	} catch (NullPointerException e) {
	    throw new ParameterNotCompatibleException();
	}
    }

    /** Initialize crypto mechanisms
    * @param sessionKey symmetric mechanism private key
    * @param mode mechanism mode
    */
    private void initCipher(String mechName, Key sessionKey, String mode) {
	try {
	    CipherIn =  Cipher.getInstance(mechName + "/" + mode + "/OneAndZeroes");
	    CipherOut = Cipher.getInstance(mechName + "/" + mode + "/OneAndZeroes");
	    CipherIn.initDecrypt(sessionKey);
	    CipherOut.initEncrypt(sessionKey);
	    System.out.println(mechName + " instance initialized");
	} catch(Exception e) {
	    System.out.println(e);
	}
    }
    public MechanismConfiguration getDefaultConfiguration() {
	return null;
    }

    public MechanismConfiguration configDetail(JFrame parentFrame, MechanismConfiguration actualMechanismConfiguration) {
	return null;
    }

    /** encrypt ByteArray with public key
    * @param input data to encrypt
    * @param publicKey public key
    * @return byte[] encrypted data
    */
    private byte[] encrypt(byte[] input, PublicKey publicKey) {
	try {
	    Cipher cipherOut=Cipher.getInstance("RSA/ECB/OneAndZeroes");

	    byte[] rawBuffer = GWKeyExchange.DERtoRAW(publicKey.getEncoded());
	    ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(rawBuffer);
	    RawRSAPublicKey rawRSAPublicKey = new RawRSAPublicKey(byteArrayInputStream);

	    cipherOut.initEncrypt(rawRSAPublicKey);
	    byte[] ouput=cipherOut.crypt(input);
	    return ouput;
	} catch(java.lang.Exception e) {
	    System.out.println("Error in encrypt");
	    e.printStackTrace();
	}
	return null;
    }

    public byte[] getObjectAsByteArray(Object object) {
	try {
	    ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
	    ObjectOutputStream objectOutputStream = new ObjectOutputStream(byteArrayOutputStream);
	    objectOutputStream.writeObject(object);
	    objectOutputStream.flush();
	    byte[] b = byteArrayOutputStream.toByteArray();
	    System.out.println("getObjectAsByteArray lgt.: "+b.length);
	    return b;
	} catch (Exception e) {
	    System.out.println("Error while transforming into ByteArray.");
	    e.printStackTrace();
	    return null;
	}
    }
}
