package smi.user;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.util.Locale;
import java.util.ResourceBundle;
import java.io.File;
import java.io.FilenameFilter;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import ssonet.Preload;

/**
 * Title:        SSONET_N
 * Description:  SSONET - Sicherheit und Schutz in offenen Netzen
 * fr Client-Server-Verbindungen
 * Copyright:    Copyright (c) 2000
 * Company:      TU Dresden
 * @author Sebastian Clau
 * @version 1.0
 */

public class UserSelectDialog extends JDialog implements ActionListener {
    private ResourceBundle resource;
    private Locale locale;

    private JLabel userLabel;
    private JButton newUserButton, propertiesButton, selectButton, abortButton;
    private JList userList;

    private File[] users;

    private String selectedUser;

    private boolean debug;

    private class UserDirFilter implements FilenameFilter{
	public UserDirFilter(){};
	public boolean accept(File dir, String name){
	    File userFile = new File(dir.getPath() + "/"+ name);
	    if(!userFile.exists()) return false;
	    if(!userFile.isDirectory()) return false;
	    if(!userFile.canWrite()) return false;
	    return true;
	}
    }

    /**
     * Create the dialog
     */
    public UserSelectDialog(Dialog owner, boolean modal, Locale locale){
	super(owner, modal);

	// debug-mode
	try{
	    debug = false;
	    if(System.getProperty("debug") != null){
		debug = true;
	    }
	}
	catch (Exception e){};

	this.locale = locale;
	selectedUser = null;
	// Resourcen laden
	resource = ResourceBundle.getBundle("smi.baseConf.locale.Resources", locale);

	// Fenster aufbauen
	setTitle(resource.getString("SELECT USER"));

	getContentPane().setLayout(new BorderLayout());

	JPanel listPanel = new JPanel();
	listPanel.setBorder(new TitledBorder(LineBorder.createBlackLineBorder(), resource.getString("USERS")));
	GridBagLayout gl = new GridBagLayout();
	GridBagConstraints gc = new GridBagConstraints();
	listPanel.setLayout(gl);
	getContentPane().add(listPanel, BorderLayout.CENTER);

	userList = new JList();
	userList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	updateList();
	JScrollPane scrollPane = new JScrollPane(userList, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
	    JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	gc.gridx = 0;
	gc.gridy = 0;
	gc.weightx = 1;
	gc.weighty = 1;
	gc.gridwidth = 2;
	gc.insets = new Insets(5,5,5,5);
	gc.fill = GridBagConstraints.BOTH;
	gc.anchor = GridBagConstraints.NORTH;
	gl.setConstraints(scrollPane, gc);
	listPanel.add(scrollPane);

	propertiesButton = new JButton(resource.getString("USER PROPERTIES"));
	gc.gridwidth = 1;
	gc.gridy = 1;
	gc.weighty = 0;
	gc.weightx = 0;
	gc.fill = GridBagConstraints.NONE;
	gc.anchor = GridBagConstraints.SOUTHWEST;
	gl.setConstraints(propertiesButton, gc);
	listPanel.add(propertiesButton);

	newUserButton = new JButton(resource.getString("CREATE USER"));
	gc.gridx = 1;
	gc.anchor = GridBagConstraints.SOUTHEAST;
	gl.setConstraints(newUserButton, gc);
	listPanel.add(newUserButton);

	JPanel buttonPanel = new JPanel();
	buttonPanel.setLayout(gl);
	getContentPane().add(buttonPanel, BorderLayout.SOUTH);

	selectButton = new JButton(resource.getString("OK"));
	gc.gridx = 0;
	gc.gridy = 0;
	gc.anchor = GridBagConstraints.CENTER;
	gl.setConstraints(selectButton, gc);
	buttonPanel.add(selectButton);

	abortButton = new JButton(resource.getString("CANCEL"));
	gc.gridx = 1;
	gl.setConstraints(abortButton, gc);
	buttonPanel.add(abortButton);

	newUserButton.addActionListener(this);
	propertiesButton.addActionListener(this);
	selectButton.addActionListener(this);
	abortButton.addActionListener(this);

	pack();
    }

    /*
    * Actionevents auswerten
    */
    public void actionPerformed(ActionEvent ae){
	if(ae.getSource() == newUserButton){
	    // neuen Nutzer anlegen
	    UserPropertiesDialog upd = new UserPropertiesDialog(this, true, locale, null, true);
	    upd.show();
	    updateList();
	}
	else if(ae.getSource() == propertiesButton){
	    // Nutzereigenschaften aendern
	    UserPropertiesDialog upd = new UserPropertiesDialog(this, true, locale,
		users[userList.getSelectedIndex()].getName(), false);
	    upd.show();
	}
	else if(ae.getSource() == selectButton){
	    // Nutzer auswaehlen und Dialog schliessen
	    if(!userList.isSelectionEmpty()){
		selectedUser = users[userList.getSelectedIndex()].getName();
		this.dispose();
	    }
	}
	else if(ae.getSource() == abortButton){
	    // Nutzerauswahl zuruecksetzen und Dialog schliessen
	    selectedUser = null;
	    this.dispose();
	}
    }

    /**
     * Verzeichnisnamen des ausgewahlten Nutzerverzeichnisses ausgeben.
     */
    public String getSelectedUser(){
	return selectedUser;
    }

    /**
     * Nutzerliste aus Dateisystem ermitteln und in Liste eintragen.
     */
    private void updateList(){
	//Nutzer suchen (Zunchst werden sie anhand des Verzeichnisnamens
	// referenziert, es wird angenommen, da nur Nutzerverzeichnisse
	// dort liegen
	// XXX spter verbessern (Filter einfhren))
	File userDir = new File(System.getProperty("ssonet_cfg")+"users");
	try{
	    // remember selection
	    String selectedName = null;
	    if(userList.getSelectedIndex() >= 0){
		selectedName = users[userList.getSelectedIndex()].getName();
	    }
	    // Check userdirectory
	    if(!userDir.exists()) throw new Exception(resource.getString("USER DIRECTORY")+"\n\""
		+System.getProperty("ssonet_cfg")+"users\"\n "+resource.getString("DOES NOT EXIST"));
	    if(!userDir.isDirectory()) throw new Exception("\""+ System.getProperty("ssonet_cfg")+"users\"\n"
		+ resource.getString("IS NOT A DIRECTORY")+".");
	    if(!userDir.canRead()) throw new Exception(resource.getString("USER DIRECTORY")+"\n\""
		+resource.getString("IS UNREADABLE")+".");
	    // get userlist
	    users = userDir.listFiles(new UserDirFilter());
	    if(debug) System.out.println("Laenge userliste:"+users.length);
	    String[] userDirNames = new String[users.length];
	    int sIndex = 0;
	    for(int i = 0; i < users.length; i++){
		userDirNames[i] = users[i].getName();
		if(selectedName != null){
		    if(selectedName.compareTo(userDirNames[i]) == 0){
			sIndex = i;
		    }
		}
	    }
	    userList.setListData(userDirNames);
	    if(sIndex < userDirNames.length){
		userList.setSelectedIndex(sIndex);
	    }
	}
	catch(Exception e){
	    JOptionPane.showMessageDialog(this, resource.getString("ERROR READING USERS")+":\n\n"
		+e.getMessage(),
		resource.getString("ERROR"), JOptionPane.ERROR_MESSAGE);
	}
    };

    /**
     * dummy main function for debugging
     */
    public static void main(String argv[]){
	Preload preload = new Preload();
	try{
	    preload.join();
	}catch (Exception e){}

	UserSelectDialog usd = new UserSelectDialog(new JDialog(), true, Locale.GERMAN);
	usd.show();

	System.exit(0);
    };

}