/*
 * ssonet/IdentityCheck.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package ssonet;

import java.awt.Container;
import java.awt.Font;
import java.awt.Insets;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JTextArea;

import sun.security.x509.X509Cert;

public class IdentityCheck implements ActionListener {
    private static JButton yesButton, noButton;
    private static boolean returnCode = false;
    public static JDialog dialog;
    private static long interactiveTime=0;

    /** tests interactive the identity of the communication partner by testing his/her certificate
    * @param certificateName description of the certificate
    * @param certificate communications partner certificate
    * @param parentFrame parent frame
    */

    public IdentityCheck(String certificateName, X509Cert certificate, JFrame parentFrame) {
	try	{
	    GridBagLayout gridBagLayout = new GridBagLayout();
	    GridBagConstraints gridBagConstraints = new GridBagConstraints();
	    gridBagConstraints.fill = GridBagConstraints.BOTH;
	    gridBagConstraints.insets = new Insets(5,5,5,5);

	    dialog = new JDialog(parentFrame,"Identity check for " + certificateName, true);
	    dialog.setSize(450,350);
	    Container contentPane = dialog.getContentPane();
	    contentPane.setLayout(gridBagLayout);

	    JLabel titleLabel = new JLabel("This is a public key certificate for:", JLabel.CENTER);
	    gridBagConstraints.gridx = 0;
	    gridBagConstraints.gridy = 0;
	    gridBagConstraints.gridwidth = 2;
	    gridBagConstraints.gridheight = 1;
	    gridBagConstraints.weighty = .2;
	    gridBagLayout.setConstraints(titleLabel, gridBagConstraints);
	    contentPane.add(titleLabel);

	    JTextArea subjectLabel = new JTextArea("name              :"+certificate.getSubjectName().getCommonName() + "\n" +
		"organisation      :"+certificate.getSubjectName().getOrganization() + "\n" +
		"organisation unit :"+certificate.getSubjectName().getOrganizationalUnit() + "\n" +
		"country           :"+certificate.getSubjectName().getCountry(),20,4);
	    subjectLabel.setFont(new Font("Courier New", Font.BOLD,14));
	    subjectLabel.setEditable(false);
	    gridBagConstraints.insets = new Insets(0,45,10,45);
	    gridBagConstraints.gridx = 0;
	    gridBagConstraints.gridy = 1;
	    gridBagConstraints.gridwidth = 2;
	    gridBagConstraints.gridheight = 1;
	    gridBagConstraints.weighty = 0;
	    gridBagLayout.setConstraints(subjectLabel, gridBagConstraints);
	    contentPane.add(subjectLabel);

	    JLabel middleLabel = new JLabel("It was issued by:", JLabel.CENTER);
	    gridBagConstraints.insets = new Insets(5,5,5,5);
	    gridBagConstraints.gridx = 0;
	    gridBagConstraints.gridy = 2;
	    gridBagConstraints.gridwidth = 2;
	    gridBagConstraints.gridheight = 1;
	    gridBagConstraints.weighty = .2;
	    gridBagLayout.setConstraints(middleLabel, gridBagConstraints);
	    contentPane.add(middleLabel);

	    JTextArea issuerLabel = new JTextArea("name              :"+certificate.getIssuerName().getCommonName() + "\n" +
		"organisation      :"+certificate.getIssuerName().getOrganization() + "\n" +
		"organisation unit :"+certificate.getIssuerName().getOrganizationalUnit() + "\n" +
		"country           :"+certificate.getIssuerName().getCountry(),20,4);
	    issuerLabel.setFont(new Font("Courier New", Font.BOLD,14));
	    issuerLabel.setEditable(false);
	    gridBagConstraints.insets = new Insets(0,45,10,45);
	    gridBagConstraints.gridx = 0;
	    gridBagConstraints.gridy = 3;
	    gridBagConstraints.gridwidth = 2;
	    gridBagConstraints.gridheight = 1;
	    gridBagConstraints.weighty = 0;
	    gridBagLayout.setConstraints(issuerLabel, gridBagConstraints);
	    contentPane.add(issuerLabel);

	    JLabel questionLabel = new JLabel("Is this the expected certificate?", JLabel.CENTER);
	    gridBagConstraints.insets = new Insets(5,5,5,5);
	    gridBagConstraints.gridx = 0;
	    gridBagConstraints.gridy = 4;
	    gridBagConstraints.gridwidth = 2;
	    gridBagConstraints.gridheight = 1;
	    gridBagConstraints.weighty = .2;
	    gridBagLayout.setConstraints(questionLabel, gridBagConstraints);
	    contentPane.add(questionLabel);

	    gridBagConstraints.insets = new Insets(5,25,15,25);

	    yesButton = new JButton("Yes");
	    yesButton.addActionListener(this);
	    gridBagConstraints.gridx = 0;
	    gridBagConstraints.gridy = 5;
	    gridBagConstraints.gridwidth = 1;
	    gridBagConstraints.gridheight = 1;
	    gridBagConstraints.weightx = .5;
	    gridBagConstraints.weighty = 0;
	    gridBagLayout.setConstraints(yesButton, gridBagConstraints);
	    contentPane.add(yesButton);

	    noButton = new JButton("No");
	    noButton.addActionListener(this);
	    gridBagConstraints.gridx = 1;
	    gridBagConstraints.gridy = 5;
	    gridBagConstraints.gridwidth = 1;
	    gridBagConstraints.gridheight = 1;
	    gridBagConstraints.weightx = .5;
	    gridBagConstraints.weighty = 0;
	    gridBagLayout.setConstraints(noButton, gridBagConstraints);
	    contentPane.add(noButton);
	}
	catch(Exception e) {
	    e.printStackTrace();
	}
    }


    /** tests interactive the identity of the communication partner by testing his/her certificate
    * @param certificateName description of the certificate
    * @param certificate communications partner certificate
    * @param parentFrame parent frame
    * @return boolean true if identity os OK, otherwise false
    */
    public static boolean getAnswer(String certificateName, X509Cert certificate, JFrame parentFrame) {
	interactiveTime = System.currentTimeMillis();
	IdentityCheck idCheck = new IdentityCheck(certificateName, certificate, parentFrame);
	idCheck.dialog.show();
	idCheck.dialog.requestFocus();
	interactiveTime=System.currentTimeMillis()-interactiveTime;
	return idCheck.returnCode;
    }

    public static long getInteractiveTime() {
	return interactiveTime;
    }

    public void actionPerformed (ActionEvent event) {
	if (event.getSource() == yesButton) {
	    returnCode = true;
	    dialog.dispose();
	}

	if (event.getSource() == noButton) {
	    returnCode = false;
	    dialog.dispose();
	}
    }
}
