/*
 * ssonet/SSONETServerConnection.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package ssonet;

import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.Socket;
import java.net.ServerSocket;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import smi.appConf.ApplicationConfiguration;
import smi.appConf.ActionConfiguration;
import smi.baseConf.BaseConfiguration;
import smi.io.IOStream;
import smi.io.ObjectIOStream;
import smi.mechanisms.MechanismConfiguration;
import smi.mechanisms.MechanismInstance;


import ssonet.SSONETStream;
import ssonet.Settings;

import sun.security.x509.X509Cert;

import state.Pad;

public class SSONETServerConnection extends SSONETConnection {
    private ServerSocket serversocket;

    /** Creates a SSONETServer. It behaves like a normal Server, that means it waits
    * for connection requests. Connection requests can be accepted.
    * @param applicationName name of the application
    * @param parentFrame parent Frame
    * @param Port for the serversocket
    * @exception IOException if an error occures, e.g. because the configuration data
    * couldnt be read or the port is already in use.
    */
    public SSONETServerConnection(String applicationName, JFrame parentFrame, int port)throws IOException {
	this.applicationName = applicationName;
	this.parentFrame = parentFrame;
	try {
	    serversocket = new ServerSocket(port);
	    writeStatus("Serversocket created.");

	    initConnection();
	} catch (Exception e) {
	    writeStatus("Error during SSONETConnection initialization");
	    e.printStackTrace();
	    throw new IOException();
	}
    }

    /** Waits for incoming connection requests. If one arrives, a negotiation
    * with the ohter side takes place. If the result is positive one can communicate
    * writeObject and readObject.
    * If the negotiation fails, an exception is thrown.
    * @return IOStream for communication
    */
    public IOStream accept() throws IOException {
	writeStatus("SSONETServerConnection starts...\n");
	System.out.println("SSONETServerConnection starts.");
	Socket s;
	try {
	    s = serversocket.accept();
	    writeStatus("Conection request.");
	    InputStream in = s.getInputStream();
	    OutputStream out = s.getOutputStream();

	    ActionConfiguration partnerActionConfiguration = ActionConfiguration.fromStream(in);
	    writeStatus("got partner configuration:\n" + partnerActionConfiguration.toString());
	    System.out.println("got partner configuration:\n" + partnerActionConfiguration.toString());
	    ActionConfiguration ownActionConfiguration =
		ApplicationConfiguration.getActionConfiguration(applicationName, partnerActionConfiguration.getID());

	    writeStatus("sending own configuration:\n" + ownActionConfiguration.toString());
	    System.out.println("sending own configuration:\n" + ownActionConfiguration.toString());
	    ownActionConfiguration.toStream(out, true);

	    SSONETNegotiate.negotiatePhaseOne(ownActionConfiguration, partnerActionConfiguration);
	    writeStatus("own configuration after phase 1:\n" + ownActionConfiguration.toString());
	    System.out.println("own configuration after phase 1:\n" + ownActionConfiguration.toString());

	    partnerActionConfiguration = ActionConfiguration.fromStream(in);
	    writeStatus("partner configuration after phase 1:\n" + partnerActionConfiguration.toString());
	    System.out.println("partner configuration after phase 1:\n" + partnerActionConfiguration.toString());

	    ownActionConfiguration.toStream(out, true);
	    PhaseTwoResult phaseTwoResult = SSONETNegotiate.negotiatePhaseTwo(ownActionConfiguration,
		partnerActionConfiguration,
		settings);
	    boolean conflictOccured = false;
	    // determining if any conflict occured....
	    for (int i = 0; i < 5; i++)
		if (phaseTwoResult.get(i) == SSONETConstants.CONFLICT) conflictOccured = true;
	    if (conflictOccured) {
		writeStatus("Negotiation not successful:\n");
		System.out.println("Negotiation not successful:");
	    }

	    // report details...
	    if (phaseTwoResult.get(SSONETConstants.CONFIDENTIALITY) == SSONETConstants.CONFLICT) {
		writeStatus("-> Conflict regarding Confidentiality!\n");
		System.out.println("-> Conflict regarding Confidentiality!");
	    }
	    if (phaseTwoResult.get(SSONETConstants.ACCOUNTABILITY_SIGN) == SSONETConstants.CONFLICT) {
		writeStatus("-> Conflict regarding Accountability!\n");
		System.out.println("-> Conflict regarding Accountability!");
	    }
	    if (phaseTwoResult.get(SSONETConstants.INTEGRITY) == SSONETConstants.CONFLICT) {
		writeStatus("-> Conflict regarding Integrity!\n");
		System.out.println("-> Conflict regarding Integrity!");
	    }
	    if (phaseTwoResult.get(SSONETConstants.ANONYMITY) == SSONETConstants.CONFLICT) {
		writeStatus("-> Conflict regarding Anonymity!\n");
		System.out.println("-> Conflict regarding Anonymity!");
	    }

	    if (conflictOccured) {
		writeStatus("-> No connection!\n");
		System.out.println("-> No connection!");
		return null;
	    }

	    ObjectIOStream objectIOStream = new ObjectIOStream(in, out);

	    PhaseTwoConfiguration partnerConfigurationPhase2 = (PhaseTwoConfiguration)objectIOStream.readObject();
	    System.out.println(partnerConfigurationPhase2.toString());
	    PhaseTwoConfiguration ownConfigurationPhase2 = new PhaseTwoConfiguration(settings);
	    if(settings.securityGoal[SSONETConstants.CONFIDENTIALITY]) {
		MechanismConfiguration[] conf = new MechanismConfiguration[ownConfidencialityPreferenceList.length];
		for(int i = 0; i < ownConfidencialityPreferenceList.length; i++) {
  //nderung
		    conf[i] = MechanismInstance.getMechanismConfiguration(SSONETConstants.CONFIDENTIALITY,
			ownConfidencialityPreferenceList[i].string);
		}
		ownConfigurationPhase2.setConfidentiality(ownConfidencialityPreferenceList, conf);
	    }
	    if(settings.securityGoal[SSONETConstants.ACCOUNTABILITY_SIGN]) {
		MechanismConfiguration[] conf = new MechanismConfiguration[ownAccountabilityPreferenceList.length];
		for(int i = 0; i < ownAccountabilityPreferenceList.length; i++) {
  //nderung
		    conf[i] = MechanismInstance.getMechanismConfiguration(SSONETConstants.ACCOUNTABILITY_SIGN,
			ownAccountabilityPreferenceList[i].string);
		}
		ownConfigurationPhase2.setAccountability(ownAccountabilityPreferenceList, conf);
	    }
	    if(settings.securityGoal[SSONETConstants.INTEGRITY]) {
		MechanismConfiguration[] conf = new MechanismConfiguration[ownIntegrityPreferenceList.length];
		for(int i = 0; i < ownIntegrityPreferenceList.length; i++) {
  //nderung
		    conf[i] = MechanismInstance.getMechanismConfiguration(SSONETConstants.INTEGRITY,
			ownIntegrityPreferenceList[i].string);
		}
		ownConfigurationPhase2.setIntegrity(ownIntegrityPreferenceList, conf);
	    }

	    objectIOStream.writeObject(ownConfigurationPhase2);

	    IOStream ioStream = SSONETNegotiate.negotiatePhaseThree(ownConfigurationPhase2,
		partnerConfigurationPhase2,
		objectIOStream,
		parentFrame,
		trustCenterTestKey,
		ownTestCertificate,
		ownCipherCertificate,
		true);
	    if(ioStream == null) {
		System.out.println("socket.close()");
		s.close();
		throw new IOException();
	    }

	    System.out.println(Integer.toString(partnerActionConfiguration.getID()));
	    return new SSONETStream(partnerActionConfiguration.getID(), ioStream);

	} catch(Exception e) {
	    writeStatus(e.toString());
	    e.printStackTrace();
	    throw new IOException();
	}
    }

    public void close() {
	try {
	    serversocket.close();
	} catch(Exception e) {
	    System.out.println("Error in client_server.Server.init");
	    e.printStackTrace();
	}
    }

    public Settings getSettings() {
	return settings;
    }
}
