/*
 * Generator/CertificateGenerator.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package trustcenter;
import cryptix.provider.rsa.RawRSAPublicKey;

import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import sun.security.util.DerValue;
import sun.security.util.DerInputStream;
import sun.security.util.DerOutputStream;
import sun.security.util.BigInt;
import sun.security.x509.AlgorithmId;
import sun.security.x509.X509Cert;
import sun.security.x509.X509Key;
import sun.security.x509.X500Name;
import sun.security.x509.X500Signer;

public class CertificateGenerator {

    private static X509Cert trustCenterCertificate;
    private static X500Signer signer;
    private static X509Key publicKey;

    public CertificateGenerator(String issuer, String issuerOU, String issuerO, String issuerCountry,
	String validity, String certName, String pubKeyName, String privKeyName, String issuerCertName){
	try {
	    X500Name subjectName = new X500Name(issuer, issuerOU, issuerO, issuerCountry);

	    GregorianCalendar gregorianCalendar = (GregorianCalendar)Calendar.getInstance();
	    Date notBefore = gregorianCalendar.getTime();
	    gregorianCalendar.add(Calendar.DAY_OF_YEAR, new Integer(validity).intValue());
	    Date notAfter = gregorianCalendar.getTime();

	    System.out.println("reading public key from "+pubKeyName+"\n");
	    FileInputStream fileInputStream = new FileInputStream(pubKeyName);
	    ObjectInputStream objectInputStream = new ObjectInputStream(fileInputStream);
	    publicKey = (X509Key)objectInputStream.readObject();
	    objectInputStream.close();
	    fileInputStream.close();

	    System.out.println("reading trustcenter private key from "+privKeyName);
	    fileInputStream = new FileInputStream(privKeyName);
	    objectInputStream = new ObjectInputStream(fileInputStream);
	    PrivateKey privateKey = (PrivateKey)objectInputStream.readObject();
	    objectInputStream.close();
	    fileInputStream.close();

	    System.out.println("create certificate.");

	    X509Cert certificate = new X509Cert(subjectName, publicKey, notBefore, notAfter);
	    if (issuerCertName.equals("XXX")) {
		System.out.println("create selfsigned certificate");
		signer = certificate.getSigner(new AlgorithmId(AlgorithmId.DSA_oid), privateKey);
	    }
	    else {
		System.out.println("reading issuer certificate "+issuerCertName);
		fileInputStream = new FileInputStream(issuerCertName);
		trustCenterCertificate = new X509Cert();
		trustCenterCertificate.decode(fileInputStream);
		fileInputStream.close();
		signer = trustCenterCertificate.getSigner(new AlgorithmId(AlgorithmId.DSA_oid), privateKey);
	    }

	    System.out.println("sign certificate");
	    X509Cert signedCertificate = new X509Cert(certificate.encodeAndSign(new BigInt(1234567890), signer));
	    System.out.println("Certifikat:\n"+signedCertificate.toString());

	    System.out.println("writing Certificate into File: "+certName);
	    FileOutputStream fileOutputStream = new FileOutputStream(certName);
	    signedCertificate.encode(fileOutputStream);
	    fileOutputStream.close();
	}
	catch(java.lang.Exception ex) {
	    System.out.println(ex);
	}
    }

    private static int bitToByte(int numberOfBits) {
	if ((numberOfBits%8)==0) return (numberOfBits/8);
	else return ((numberOfBits/8)+1);
    }

    private static String toHex(byte b) {
	int i=(b>>4)&0x0F;
	String tmp;
	if(i<10) tmp=Integer.toString(i);
	else tmp=new Character((char)('A'+i-10)).toString();
	i=b&0x0F;
	if(i<10) tmp+=Integer.toString(i);
	else tmp+=new Character((char)('A'+(i-10))).toString();
	return tmp;
    }

    public static void main(String[] argv) {
	if(argv.length < 8){
	    System.out.println("Usage: CertificateGenerator <issuer> <issuer-OrgUnit> \n"
		+"       <issuer-Org> <issuer-country> <validity in days> \n"
		+"       <certificate filename> <public key to certifie> <issuers private key>"
		+"       <issuer certificate (\"XXX\" for selfsigned certificate)>");
	    System.exit(0);
	}
	new CertificateGenerator(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6],
	    argv[7], argv[8]);
    }
}
